######################## Chapter 10 Script #################
############################################################

#Clear workspace
rm(list=ls(all=T))

########################
# 10.2 Improving Fit ###
########################

#.........................................................
# 10.2.1 Power transformations of explanatory variables ..
#.........................................................

##############################################################
#Execute this command to duplicate contents of this section ##
##############################################################

source("C:/Docs/RCompanion/Chapter10/Data/Data10x01.R")

###################################################################
# Data generation code ..
#........................

x1<-runif(35,1,5);x2<-runif(35,0,1);x3<-runif(35,1,10)
y<-3-2*x1^2+sqrt(x2)-1/2*x3^(0.25)+rnorm(35,mean=0,sd=.1)
BoxTidData<-data.frame(cbind(y,x1,x2,x3)); rm(y,x1,x2,x3)

#Save if desired
dump("BoxTidData","C:/Docs/RCompanion/Chapter10/Data/Data10x01.R")
###################################################################

#............
# Fig 10.1 ..
#............

#Fit untransformed model
First.mod<-lm(y~x1+x2+x3,BoxTidData)
win.graph(width=6,height=3.5); par(mfrow=c(1,2),ps=10,cex=.75)
with(First.mod,plot(fitted.values,residuals,
	xlab="Fitted Values",ylab="Residuals"))
with(First.mod,qqnorm(residuals,main=NULL))
par(mfrow=c(1,1))

#Box-Tidwell procedure
library(car)
with(BoxTidData,boxTidwell(y~x1+x2+x3))
detach(package:car);detach(package:MASS)
detach(package:nnet);detach(package:survival)
detach(package:splines)

#Fit transformed model
Second.mod<-lm(y~I(x1^2)+I(x2^(0.3))+I(x3^(-0.6)),BoxTidData)

#.............
# Fig 10.2 ...
#.............

win.graph(width=6,height=3.5); par(mfrow=c(1,2),ps=10,cex=.75)
with(Second.mod,plot(fitted.values,residuals,
	xlab="Fitted Values",ylab="Residuals",las=1))
with(Second.mod,qqnorm(residuals,main=NULL,las=1))
par(mfrow=c(1,1))

#Fit another transformed model
Third.mod<-lm(y~x1+I(x1^2)+I(x2^(0.3))+I(x3^(-0.6)),BoxTidData)
win.graph(width=6,height=3.5); par(mfrow=c(1,2),ps=10,cex=.75)
with(Third.mod,plot(fitted.values,residuals,
        xlab="Fitted Values",ylab="Residuals",las=1))
with(Third.mod,qqnorm(residuals,main=NULL))
par(mfrow=c(1,1))

#...............................................................
# 10.2.2 Transforming both response and explanatory variables ..
#...............................................................

#Clear workspace
rm(list=ls(all=T))

##############################################################
#Execute this command to duplicate contents of this section ##
##############################################################

source("C:/Docs/RCompanion/Chapter10/Data/Data10x02.R")

########################################################################
# Data generation code ..
#........................

x1<-runif(35,0,1);x2<-runif(35,0,1)
y<-(2.3+1.25*x1^2-1.5*sqrt(x2)+rnorm(35,mean=0,sd=0.15))^2
XYTransData<-data.frame(y,x1,x2); rm(y,x1,x2)

#Save if desired
dump("XYTransData","C:/Docs/RCompanion/Chapter10/Data/Data10x02.R")
########################################################################


#............
# Fig 10.3 ..
#............

#Fit the untransformed model and plot diagnostic plots
First.mod<-lm(y~x1+x2,XYTransData)
win.graph(width=6,height=3.5); par(mfrow=c(1,2),ps=10,cex=.75)
with(First.mod,plot(fitted.values,residuals,
	xlab="Fitted Values",ylab="Residuals"))
with(First.mod,qqnorm(residuals,main=NULL))
par(mfrow=c(1,1))

#............
# Fig 10.4 ..
#............

#Fit the transformed model and plot diagnostic plots
Second.mod<-lm(sqrt(y)~x1+x2,XYTransData)
win.graph(width=6,height=3.5); par(mfrow=c(1,2),ps=10,cex=.75)
with(Second.mod,plot(fitted.values,residuals,
	xlab="Fitted Values",ylab="Residuals"))
with(Second.mod,qqnorm(residuals,main=NULL))
par(mfrow=c(1,1))

#............
# Fig 10.5 ..
#............

attach(XYTransData)
win.graph(width=6,height=6); par(mfrow=c(2,2),ps=10,cex=.75)
with(Second.mod,plot(residuals~x1))
with(Second.mod,plot(residuals~x2))
plot(sqrt(y)~x1,ylab=expression(sqrt(y)))
plot(sqrt(y)~x2,ylab=expression(sqrt(y)))
detach(XYTransData)
par(mfrow=c(1,1))

#.........................
# Box-Tidwell procedure ..
#.........................

library(car)
with(XYTransData,boxTidwell(sqrt(y)~x1+x2))
detach(package:car);detach(package:MASS)
detach(package:nnet);detach(package:survival)
detach(package:splines)

lm(sqrt(y)~I(x1^2)+sqrt(x2),XYTransData)$coefficients

##########################
# Further explorations ###
##########################
Third.mod<-lm(sqrt(y)~x1+I(x1^2)+sqrt(x2),XYTransData)

attach(XYTransData)
win.graph(width=6,height=8); par(mfrow=c(3,2),ps=10,cex=.75)
with(Third.mod,plot(fitted.values,residuals,
	xlab="Fitted Values",ylab="Residuals"))
with(Third.mod,qqnorm(residuals,main=NULL))
with(Third.mod,plot(residuals~x1))
with(Third.mod,plot(residuals~x2))
plot(sqrt(y)~I(x1^2),ylab=expression(sqrt(y)),xlab=expression(x1^2))
plot(sqrt(y)~sqrt(x2),ylab=expression(sqrt(y)),xlab=expression(sqrt(x2)))
detach(XYTransData)
par(mfrow=c(1,1))

summary(Third.mod)

#####################################
# 10.3 Normalizing transformations ##
#####################################

#............
# Fig 10.6 ..
#............

library(MASS)
win.graph(width=6,height=3.5); par(mfrow=c(1,2),ps=10,cex=.75)
boxcox(lm(y~x1+x2,XYTransData))
boxcox(lm(y~x1+x2,XYTransData),lambda=seq(.3,.55,.01))
par(mfrow=c(1,1))
detach(package:MASS)

###############################
# 10.5 Polynomial regression ##
###############################

#Clear workspace
rm(list=ls(all=T))

###############################################################
#Execute these command to duplicate contents of this section ##
###############################################################

source("C:/Docs/RCompanion/Chapter10/Data/Data10x01.R")
source("C:/Docs/RCompanion/Chapter10/Data/Data10x02.R")

#............
# Fig 10.7 ..
#............

#Fit BoxTidData to polynomial model
Poly.mod<-lm(y~x1+x2+x3+I(x1*x2)+I(x1*x3)+I(x2*x3)+
	I(x1^2)+I(x2^2)+I(x3^2),BoxTidData)

#Get diagnostic plots
win.graph(width=6,height=3.5); par(mfrow=c(1,2),ps=10,cex=.75)
with(Poly.mod,plot(fitted.values,residuals,
	xlab="Fitted Values",ylab="Residuals",las=1))
with(Poly.mod,qqnorm(residuals,main=NULL,las=1))
par(mfrow=c(1,1))

#Stepwise variable selection
round(step(Poly.mod,trace=0)$coefficients,2)

#............
# Fig 10.8 ..
#............

#Fit XYTransData to polynomial model 
Poly.mod2<-lm(sqrt(y)~x1+x2+I(x1^2)+I(x1*x2)+I(x2^2)
	+I(x1^3)+I(x1^2*x2)+I(x1*x2^2)+I(x2^3),XYTransData)

#Get diagnostic plots
win.graph(width=6,height=3.5); par(mfrow=c(1,2),ps=10,cex=.75)
with(Poly.mod2,plot(fitted.values,residuals,
	xlab="Fitted Values",ylab="Residuals",las=1))
with(Poly.mod2,qqnorm(residuals,main=NULL,las=1))
par(mfrow=c(1,1))

#Stepwise variable selection
round(step(Poly.mod2,trace=0)$coefficients,2)

#################################################
# 10.8 For the curious - box-tidwell revisited ##
#################################################

#Initialize the powers
l0<-c(1,1,1)
#Define the transformation function w
w<-function(x,l){if (l!=0){return(x^l)
			}else{return(log(x))}}
#Define the z function
z<-function(w){return(w*log(w))}
#Start initial step
mod0<-lm(y~I(w(x1,l0[1]))+
		I(w(x2,l0[2]))+I(w(x3,l0[3])),BoxTidData)
#Extract parameter beta estimates
b<-mod0$coefficients[-1]
#Fit the big model
mod1<-lm(y~I(w(x1,l0[1]))+I(w(x2,l0[2]))+
		I(w(x3,l0[3]))+I(z(w(x1,l0[1])))+
			I(z(w(x2,l0[2])))+I(z(w(x3,l0[3]))),BoxTidData)
#Extract the gamma parameter estimates
g<-mod1$coefficients[5:7]
#Get updated lambdas
l1<-l0*(g/b+1)
#Fit the updated transformed model
mod3<-lm(y~I(w(x1,l1[1]))+I(w(x2,l1[2]))+I(w(x3,l1[3])),BoxTidData)
#Compute magnitude of change in lambdas, set loop counter
dif<-abs(sqrt(sum(l0-l1)^2)); k<-1
#If change in lambdasabove tolerance, begin loop
while ((k<=25)&(dif>.001)){
#Get first stage model, betas, and lambdas
mod0<-mod3; l0<-l1; b<-mod0$coefficients[-1]
#Fit the updated big model
mod1<-lm(y~I(w(x1,l0[1]))+I(w(x2,l0[2]))+I(w(x3,l0[3]))+
		I(z(w(x1,l0[1])))+I(z(w(x2,l0[2])))+I(z(w(x3,l0[3]))),BoxTidData)
#Get new gammas
g<-mod1$coefficients[5:7]
#Update the lambdas
l1<-l0*(g/b+1)
#Fit the updated transformed model
mod3<-lm(y~I(w(x1,l1[1]))+I(w(x2,l1[2]))+I(w(x3,l1[3])),BoxTidData)
#Compute magnitude of change in lambdas, set loop counter
dif<-abs(sqrt(sum(l0-l1)^2)); k<-k+1}
#Print results
names(l1)<-c("Lambda 1","Lambda 2","Lambda 3")
l1;print(paste("Iterations = ",k-1,sep=""),quote=F)

