######################## Chapter 11 Script #################
############################################################

#Clear workspace
rm(list=ls(all=T))

##############################################################
#Execute this command to duplicate contents of this Chapter ##
##############################################################

source("C:/Docs/RCompanion/Chapter11/Data/Data11x01.R")

#To define a variable as a factor, execute
OneWayData$Tr<-factor(OneWayData$Tr)

###################################
# 11.2 Exploratory Data Analysis ##
###################################

#Overall summary
summary(OneWayData)
#Two ways to get summaries by factors
by(OneWayData,OneWayData$Tr,summary)
with(OneWayData,tapply(y,Tr,summary))

#Additional capabilities of by and tapply functions
by(OneWayData,OneWayData$Tr,length)
with(OneWayData,tapply(y,Tr,length))

#............
# Fig 11.1 ..
#............

win.graph(width=6,height=3.5); par(mfrow=c(1,2),ps=10,cex=.75)
with(OneWayData,stripchart(y~Tr,
	method="stack",vertical=T,xlab="Tr",ylab="y"))
with(OneWayData,plot(y~Tr,ylab="y",xlab="Tr"))
par(mfrow=c(1,1))

####################################
# 11.3 Model Construction and Fit ##
####################################

#.............................
# Fitting the default model ..
#.............................

#Setting the baseline treatment and contrast type
contr<-contr.treatment(5,base=5)
OneWayData$Tr<-C(OneWayData$Tr,contr,how.many=4)
options(contrasts=c("contr.treatment","contr.poly"))

#Fitting the model and obtaining detailed summary object
tc.mod<-aov(y~Tr,OneWayData)
tc.sum<-summary.lm(tc.mod)

#Get ANOVA table to assess fit
summary(tc.mod)

#####################
# 11.4 Diagnostics ##
#####################

#Extract relevant statistics
y.hat<-fitted.values(tc.mod)
e<-residuals(tc.mod)
r<-e/(tc.sum$sigma*sqrt(1-hatvalues(tc.mod)))
d<-rstudent(tc.mod)

#........................................
# 11.4.1 Constant variances assumption ..
#........................................

#............
# Fig 11.2 ..
#............

win.graph(width=6,height=3.5); par(mfrow=c(1,2),ps=10,cex=.75)
plot(e~y.hat,xlab="Fitted values",ylab="Residuals")
stripchart(e~Tr,OneWayData,method="jitter",vertical=T,
	ylab="Residuals",xlab="Treatments")
par(mfrow=c(1,1))

#Brown-Forsyth test
library(HH)
hov(y~Tr,OneWayData)
detach(package:HH);detach(package:lattice);detach(package:grid)
detach(package:multcomp);detach(package:mvtnorm);detach(package:survival)
detach(package:splines);detach(package:leaps)

#...............................
# 11.4.2 Normality assumption ..
#...............................

#............
# Fig 11.3 ..
#............

win.graph(width=3,height=3.5); par(ps=10,cex=.75)
qq<-qqnorm(r,main=NULL)
abline(a=0,b=1,lty=3)
with(qq,identify(x,y))

#QQ Normal probability correlation coefficient test
source("C:/Docs/RCompanion/Chapter6/Functions/qqcortest.R")
qq.cortest(r,0.05)

#Shapiro-Wilk test of normality
shapiro.test(r)

#.................................................
# 11.4.4 The presence and influence of outliers ..
#.................................................

#............
# Fig 11.4 ..
#............

win.graph(width=4,height=3.5); par(ps=10,cex=.75)
plot(d~y.hat,ylim=c(-3.5,3.5),
	ylab=expression(hat(d)),xlab=expression(hat(y)))
m<-ceiling(0.1*length(d)); df<-length(d)-5-1
a<-0.05/m; cv<-qt(a/2,df,lower.tail=F)
abline(h=c(-cv,0,cv),lty=c(3,1,3))
identify(y.hat,d)

#............Surplus code.................
#Use of function outlierTest
library(car)
outlierTest(tc.mod)
detach(package:car);detach(package:nnet);
detach(package:MASS);detach(package:survival)
detach(package:splines)

#Curiosity check of outlierTest output
2*pt(max(abs(d)),length(d)-5-1,lower.tail=F)
2*pt(max(abs(d)),length(d)-5-1,lower.tail=F)*length(d)

#Influence measures
influence.measures(tc.mod)$is.inf[c(42,45),]
#.........................................................


###################################################
# 11.5 Pairwise comparisons of Treatment Effects ##
###################################################

#...................................
# 11.5.1 With a control treatment ..
#...................................

tc.sum

#......................................
# 11.5.2 Without a control treatment ..
#......................................

#...................................
# 11.5.2.1 Single pairwise t-test ..
#...................................

with(OneWayData,pairwise.t.test(y,Tr,p.adj="none"))

#..........................................
# 11.5.2.2 Simultaneous pairwise t-tests ..
#..........................................

with(OneWayData,pairwise.t.test(y,Tr,p.adj="bonf"))

#..................................
# 11.5.2.3 Tukey's HSD procedure ..
#..................................

TukeyHSD(tc.mod,ordered=T)

###################################
# 11.6 Testing General Contrasts ##
###################################

#Define the contrast
contr<-rbind("m1-.25m2-.25m3-.25m4-.25m5"=
                    c(1,-.25,-.25,-.25,-.25))
#Load package multcomp
library(multcomp)
#Create general linear hypothesis test object
glh.model<-glht(tc.mod,linfct=mcp(Tr=contr))
#Obtain test results with an F-test
summary(glh.model,Ftest())

#Obtain test results with t-tests
summary(glh.model)

#Detach multcomp and accompanying packages
detach(package:multcomp);detach(package:mvtnorm)
detach(package:survival);detach(package:splines)

#############################################
# 11.7 Alternative Variable Coding Schemes ##
#############################################

#................................
# 11.7.1 Treatment means model ..
#................................

#Fit model and obtain summary
tm.mod<-aov(y~Tr-1,OneWayData)
tm.sum<-summary.lm(tm.mod)

#Get Anova table - which, by the way - is no use
summary(tm.mod)

#Test difference in means/effects
red.model<-aov(y~1,OneWayData)
anova(red.model,tm.mod)

#..................................
# 11.7.2 Treatment effects model ..
#..................................

#....................................
# 11.7.2.1 Weighted mean condition ..
#....................................

#Create the K (contrast) matrix
a<-numeric(4)
attach(OneWayData)
for (i in 1:4) {a[i]<-length(Tr[Tr==i])/length(Tr[Tr==5])}
detach(OneWayData)
K<-matrix(c(1,0,0,0,-a[1],
            0,1,0,0,-a[2],
            0,0,1,0,-a[3],
            0,0,0,1,-a[4]),nrow=5,ncol=4)
OneWayData$Tr<-C(OneWayData$Tr,K,how.many=4)

#Fit the model, etc.
wm.mod<-aov(y~Tr,OneWayData)
wm.sum<-summary.lm(wm.mod)

#Get ANOVA table
summary(wm.mod)

#....................................
# 11.7.2 Unweighted mean condition ..
#....................................

#Create the K (contrast) matrix
K<-contr.sum(5)
OneWayData$Tr<-C(OneWayData$Tr,K,how.many=4)

#Fit the model, etc
sc.mod<-aov(y~Tr,OneWayData)
sc.sum<-summary.lm(sc.mod)

#Get the ANOVA table
summary(sc.mod)

#########################
# 11.8 For the Curious ##
#########################

#................................................
# 11.8.1 Interval estimates of treatment means ..
#................................................

#........................
# 11.8.1.1 t-intervals ..
#........................

#One at a time interval(s)
confint(tm.mod)
#Simultaneous intervals
confint(tm.mod,level=1-.05/5)

#................................
# 11.8.1.2 Scheffe F-intervals ..
#................................

#First clean up the workspace
rm(list=ls(all=T))
#Reload OneWayData
source("C:/Docs/RCompanion/Chapter11/Data/Data11x01.R")
#Then refit the treatment coded model etc.
#Setting the baseline treatment and contrast type
contr<-contr.treatment(5,base=5)
OneWayData$Tr<-C(OneWayData$Tr,contr,how.many=4)
options(contrasts=c("contr.treatment","contr.poly"))

#Fitting the model and obtaining detailed summary object
tc.mod<-aov(y~Tr,OneWayData)
tc.sum<-summary.lm(tc.mod)


#Attach tc.sum for convenience
attach(tc.sum)
#Initialize storage locations
mus<-numeric(5);S.ers<-numeric(5)
lwr<-numeric(5); upr<-numeric(5)
#List names of entries
labs<-c("mu1","mu2","mu3","mu4","mu5")
#Get degrees of freedom for F-statistic
df1<-df[1];df2<-df[2]
#Calculate "critical value"
cv<-sqrt(df1*qf(.05,df1,df2,lower.tail=F))
#Use for loop to get stats for first four parameters
for (i in 1:4) {
#Get estimated treatment means
mus[i]<-coefficients[1,1]+coefficients[i+1,1]
#Get corresponding standard errors, close for-loop
S.ers[i]<-sigma*sqrt(cov.unscaled[1,1]+
     cov.unscaled[i+1,i+1]+2*cov.unscaled[1,i+1])}
#Get fifth sample mean and corresponding standard error, then detach tc.sum
mus[5]<-coefficients[1,1]
S.ers[5]<-sigma*sqrt(cov.unscaled[1,1])
detach(tc.sum)
#Start loop to get interval estimates
for (j in 1:5){
lwr[j]<-round(mus[j]-cv*S.ers[j],4)
upr[j]<-round(mus[j]+cv*S.ers[j],4)}
#Store results
F.Intervals<-data.frame("Est."=mus,"S.er"=S.ers,
              "Lower"=lwr,"Upper"=upr,row.names=labs)
#Output results
F.Intervals

#Cleanup workspace
ls()
rm(contr,cv,df1,df2,F.Intervals,i,j,labs,lwr,mus,S.ers,upr)
ls()

#......................................
# 11.8.2 Tukey-Kramer pairwise tests ..
#......................................

#Get sample means and sizes
mus<-with(OneWayData,tapply(y,Tr,mean))
ns<-with(OneWayData,tapply(y,Tr,length))
#Get ordering, number of samples and entry names
pos<-order(mus); p<-length(mus)
labs<-c("mu1","mu2","mu3","mu4","mu5")
#Store this stuff and clean house a bit
stuff<-data.frame(cbind(pos,mus,ns),row.names=labs)
rm(mus,pos,ns)
#Initialize comparison counter, get number of comparisons
k<-1; m<-choose(5,2)
#Get standard error and degrees of freedom
s<-tc.sum$sigma; dfs<-tc.sum$df[2]
#Get "q-critical value" for interval estimates
q<-qtukey(.05,p,dfs,lower.tail=F)/sqrt(2)
#Initialize the various storage spaces
pair<-c(rep(" ",m)); diff<-numeric(m);
lwr<-numeric(m); upr<-numeric(m)
q.stat<-numeric(m); p.value<-numeric(m)
#Attach stuff for convenience
attach(stuff)
#Begin outer (ascending) for-loop
for (i in 1:4){
    #Begin inner(descending) for-loop
    for (j in seq(5,i+1,-1)){
        #Get names of pairs being compared
        pair[k]<-paste(row.names(stuff[pos==j,]),
              "-",row.names(stuff[pos==i,]))
        #Compute difference between sample means
        diff[k]<-round(mus[pos==j]-mus[pos==i],4)
        #Compute lowerbound for interval
        lwr[k]<-round(diff[k]-
             q*s*sqrt(1/ns[pos==j]+1/ns[pos==i]),4)
        #Compute upperbound for interval
        upr[k]<-round(diff[k]+
             q*s*sqrt(1/ns[pos==j]+1/ns[pos==i]),4)
        #Get q-statistic
        q.stat[k]<-round(diff[k]/
             (s*sqrt((1/ns[pos==j]+1/ns[pos==i])/2)),4)
        #Get p-value
        p.value[k]<-round(ptukey(q.stat[k],
             p,dfs,lower.tail=F),4)
        #Update comparison counter and close both loops
        k<-k+1}}
detach(stuff)
#Store details
TK.Comps<-data.frame("Pairs"=pair,"Diff"=diff,"Lower"=lwr,
        "Upper"=upr,"q-Stat"=q.stat,"p-value"=p.value)
#tidy up
rm(i,j,k,s,p,m,dfs,q,pair,diff,lwr,upr,q.stat,p.value)

#Output results
TK.Comps

#.....................................
# 11.8.3 Scheffe's pairwise F-tests ..
#.....................................

mus<-with(OneWayData,tapply(y,Tr,mean))
ns<-with(OneWayData,tapply(y,Tr,length))
#Get number of samples and entry names - ordering not needed
p<-length(mus); labs<-c("mu1","mu2","mu3","mu4","mu5")
#Store this stuff and clean house a bit - no pos included
stuff<-data.frame(mus,ns,row.names=labs)
rm(mus,ns)
k<-1; m<-choose(5,2)
s<-tc.sum$sigma
#Get degrees of freedom for F-distribution
df1<-p-1; df2<-tc.sum$df[2]
#Get "F-critical value" for interval estimates
F.crit<-qf(.05, df1, df2, lower.tail = F)
pair<-c(rep(" ",m)); diff<-numeric(m);
lwr<-numeric(m); upr<-numeric(m)
F.stat<-numeric(m); p.value<-numeric(m)
attach(stuff)
#Begin outer loop, note change to counter
for (i in 1:(p-1)){
    #Begin inner loop, note change to counter
    for (j in (i+1):p){
        pair[k]<-paste(row.names(stuff[i,]),
              "-",row.names(stuff[j,]))
        diff[k]<-round(mus[i]-mus[j],4)
        #Compute lowerbound for F-interval
        lwr[k]<-round(diff[k]-
             s*sqrt(F.crit*(p-1)*(1/ns[i]+1/ns[j])),4)
        #Compute upperbound for F-interval
        upr[k]<-round(diff[k]+
             s*sqrt(F.crit*(p-1)*(1/ns[i]+1/ns[j])),4)
        #Get F-statistic
        F.stat[k]<-round(diff[k]^2/(s^2*(p-1)*(1/ns[i]+1/ns[j])),4)
        #Get p-value using F-distribution
        p.value[k]<-round(pf(F.stat[k],df1,df2,lower.tail=FALSE),4)
        k<-k+1}}
detach(stuff)
Scheffe.Comps<-data.frame("Pairs"=pair,"Diff"=diff,"Lower"=lwr,
        "Upper"=upr,"F-Stat"=F.stat,"p-value"=p.value)
rm(i,j,k,s,p,m,df1,df2,F.crit,pair,diff,lwr,upr,F.stat,p.value)

#Output results
Scheffe.Comps

#..............................
# 11.8.4 Brown Forsythe test ..
#..............................

Anova.bf.Ftest <-function(e,group){
#Get test data and factor names
dname<-deparse(substitute(e))
xname<-deparse(substitute(group))
#Store test data and factor levels in a data frame
e.list<-data.frame(e,group)
#Compute group medians & deviations from medians
meds<-with(e.list,tapply(e,group,median))
dev<-with(e.list,abs(e-meds[group]))
#Fit to a linear model
info<-anova(lm(dev~group,e.list))
#Extract and store results
results<-list(statistic=c(F=info[1,4]),
  parameters=c(df=info[,1]),p.value=info[1,5],
   method=paste("Brown-Forsyth test across factor",
    xname),data.name=dname,alternative=c("Variances are unequal"))
class(results)<-"htest";return(results)}

#Save the function source code
dump("Anova.bf.Ftest","C:/Docs/RCompanion/Chapter11/Functions/AnovaBFFTest.R")

#To recall, execute a command of the form
source("C:/Docs/RCompanion/Chapter11/Functions/AnovaBFFTest.R")

e<-residuals(tc.mod)
with(OneWayData,Anova.bf.Ftest(e,Tr))

#..................................................
# 11.8.5 Generating one-factor data to play with ..
#..................................................

#First clear the workspace
rm(list=ls(all=T))
#Initialize response and factor variable
y<-NULL; Tr<-NULL
#Start for-loop
for (j in 1:5) {
	#Get a cell size
	k<-trunc(runif(1,1,6))
	#Identify factor level
	Tr<-c(Tr,rep(j,k))
	#randomly assign a treatment mean
	mu<-runif(1,2,4)
	#Get "observed" responses
	y<-c(y,rnorm(k,mean=mu,sd=0.25))}
	#Repeat the above for j = 1 to 5
#Store y and Tr in a data frame
OneWayData<-data.frame(y,Tr)
#Define Tr as a factor
OneWayData$Tr<-factor(OneWayData$Tr)
#Take a look at the summary by treatments
with(OneWayData,tapply(y,Tr,summary))










