####################### Chapter 12 Script #################
############################################################

#CAUTION: Either restart R or be sure to reset
#the baseline treatment and contrast type to default
#Clear workspace
rm(list=ls(all=T))

#Setting 


##############################################################
#Execute this command to duplicate contents of this Chapter ##
##############################################################

source("C:/Docs/RCompanion/Chapter12/Data/Data12x01.R")

###################################
# 12.2 Exploratory Data Analysis ##
###################################

#Overall summary
summary(AncovaData)
#Summary by factor levels
with(AncovaData,by(AncovaData,Tr,summary))

#............
# Fig 12.1 ..
#............

win.graph(width=4,height=3.5); par(ps=10,cex=.75)
plot(y~x,AncovaData,pch=unclass(Tr),main=NULL)
legend(5,35,legend=c("Tr1","Tr2","Tr3"),pch=c(1,2,3),bty="n")

#............
# Fig 12.2 ..
#............

win.graph(width=6,height=3.5); par(mfrow=c(1,2),ps=10,cex=.75)
with(AncovaData,plot(Tr,y,xlab="Treatments",ylab="y"))
with(AncovaData,stripchart(y~Tr,vertical=T,
	method="jitter",xlab="Treatments",ylab="y"))
par(mfrow=c(1,1))

####################################
# 12.3 Model Construction and Fit ##
####################################

#Fitting the default treatment coded uncentered model, etc.
ancova.mod<-lm(y~Tr+x,AncovaData)
ancova.sum<-summary(ancova.mod)

#Coefficient estimates
ancova.mod$coefficients

#Testing the constant slope assumption
int.model<-lm(y~Tr+x+x:Tr,AncovaData)
anova(ancova.mod,int.model)

#Testing model significance
red.model<-lm(y~1,AncovaData)
anova(red.model,ancova.mod)

ancova.sum

#Testing treatment effects
reg.model<-lm(y~x,AncovaData)
anova(reg.model,ancova.mod)

#Using function ancova - caution
library(HH)
ancova(y~x+Tr,AncovaData)
ancova(y~Tr+x,AncovaData)
detach(package:HH);detach(package:lattice)
detach(package:grid);detach(package:multcomp)
detach(package:mvtnorm);detach(package:survival)
detach(package:splines);detach(package:leaps)

#####################
# 12.4 Diagnostics ##
#####################

#Gather statistics
y.hat<-fitted.values(ancova.mod)
e<-residuals(ancova.mod)
h<-hatvalues(ancova.mod)
r<-e/(ancova.sum$sigma*sqrt(1-h))
d<-rstudent(ancova.mod)

#.......................................
# 12.4.1 Constant variance assumption ..
#.......................................

#............
# Fig 12.3 ..
#............

win.graph(width=6.5,height=3)
par(mfrow=c(1,3),ps=10,cex=.75)
attach(AncovaData)
plot(e~y.hat,xlab="Fitted values",ylab="Residuals")
plot(e~x,xlab="x",ylab="Residuals")
stripchart(e~Tr,vertical=T,method="jitter",xlab="Treatment",ylab="Residuals")
detach(AncovaData)
par(mfrow=c(1,1))

#............Surplus plots mentioned using factor level as plot symbols......
win.graph(width=6.5,height=3)
par(mfrow=c(1,3),ps=10,cex=.75)
attach(AncovaData)
labels<-as.character(Tr)
plot(e~y.hat,pch=labels,xlab="Fitted values",ylab="Residuals")
plot(e~x,pch=labels,xlab="x",ylab="Residuals")
stripchart(e~Tr,vertical=T,method="jitter",xlab="Treatment",ylab="Residuals")
detach(AncovaData)
par(mfrow=c(1,1))

#Jitter plot of residuals with factor levels
plot(e~jitter(y.hat),pch=labels,xlab="Fitted values",ylab="Residuals")
#............................................................................

#Brown-Forsyth test - using hov

library(HH)
hov(e~Tr,AncovaData)
detach(package:HH);detach(package:lattice)
detach(package:grid);detach(package:multcomp)
detach(package:mvtnorm);detach(package:survival)
detach(package:splines);detach(package:leaps)

#Brown-Forsyth test - using Anova.bf.Ftest
source("C:/Docs/RCompanion/Chapter11/Functions/AnovaBFFTest.R")
with(AncovaData,Anova.bf.Ftest(e,Tr))

#...................................
# 12.4.2 The normality assumption ..
#...................................


#............
# Fig 12.4 ..
#............

win.graph(width=3,height=3.5); par(ps=10,cex=.75)
qqnorm(r,main=NULL)
abline(a=0,b=1,lty=3)

#QQ normal probability correlation coefficient test
source("C:/Docs/RCompanion/Chapter6/Functions/qqcortest.R")
qq.cortest(r,0.05)

#Shapiro-Wilk test of normality
shapiro.test(r)

#......................................
# 12.4.3 The independence assumption ..
#......................................

plot(e[-length(e)],e[-1])

#.................................................
# 12.4.4 The presence and influence of outliers ..
#.................................................

#............
# Fig 12.5 ..
#............

win.graph(width=4,height=3.5); par(ps=10,cex=.75)
plot(d~y.hat,xlab="Fitted values",
	ylab="Studentized deleted residuals",ylim=c(-3,3))
identify(y.hat,d)
m<-ceiling(.1*length(d));df<-length(d)-4-1
a<-0.05/m
cv<-qt(a/2,df,lower.tail=F)
abline(h=c(-cv,0,cv),lty=c(3,1,3))

#Looking at influence of outliers
inf.tests<-data.frame(influence.measures(ancova.mod)[2])
inf.tests[26,]

###################################################
# 12.5 Pairwise Comparisons of Treatment Effects ##
###################################################

#............
# Fig 12.6 ..
#............

win.graph(width=4,height=3.5); par(ps=10,cex=.75)
plot(y~x,AncovaData,pch=unclass(Tr),main=NULL,
	ylim=c(0,max(y)),xlim=c(0,max(x)))
legend(1,35,legend=c("Tr1","Tr2","Tr3"),pch=c(1,2,3),bty="n")
attach(ancova.mod)
b0<-coefficients[1];b1<-coefficients[4]
t2<-coefficients[2];t3<-coefficients[3]
detach(ancova.mod)
abline(a=b0,b=b1,lty=3)
abline(a=b0+t2,b=b1,lty=4)
abline(a=b0+t3,b=b1,lty=5)
abline(v=0,lty=3)
points(c(0,0,0),c(b0,b0+t2,b0+t3),pch=c(21,24,3))

#...................................
# 12.5.1 With a control treatment ..
#...................................

ancova.sum$coefficients

#......................................
# 12.5.2 Without a control treatment ..
#......................................

#Obtain pairs and their differences
Diff<-numeric(3); Pair<-c(rep(" ",3))
attach(ancova.mod)
Diff[1]<- coefficients[2];Pair[1]<-c("Tr2 - Tr1")
Diff[2]<- coefficients[3];Pair[2]<-c("Tr3 - Tr1")
Diff[3]<-coefficients[2]-coefficients[3]
Pair[3]<-c("Tr2 - Tr3")
detach(ancova.mod)
#Obtain variances and covariance
attach(ancova.sum)
var.Theta2<-sigma^2*cov.unscaled[2,2]
var.Theta3<-sigma^2*cov.unscaled[3,3]
cov.Theta23<-sigma^2*cov.unscaled[2,3]
detach(ancova.sum)
#Obtain standard errors and critical value
SE<-numeric(3)
SE[1]<-sqrt(var.Theta2);SE[2]<-sqrt(var.Theta3)
SE[3]<-sqrt(var.Theta2+var.Theta3-2*cov.Theta23)
crit.val<-qt(.05/6,26,lower.tail=F)
#Construct interval bounds, test values and p-values
LB<-numeric(3); UB<-numeric(3)
TV<-numeric(3); PV<-numeric(3)
for (i in (1:3)){
     LB[i]<-Diff[i]-crit.val*SE[i]
     UB[i]<-Diff[i]+crit.val*SE[i]
     TV[i]<-abs(Diff[i]/SE[i])
     PV[i]<-2*pt(TV[i],11,lower.tail=F)}
#Round down results for readability
for (i in (1:3)){
     Diff[i]<-round(Diff[i],3); SE[i]<-round(SE[i],3)
     LB[i]<-round(LB[i],3);UB[i]<-round(UB[i],3)
     TV[i]<-round(TV[i],3); PV[i]<-round(PV[i],3)}
#store results
results<-data.frame(cbind(Pair,Diff,SE,LB,UB,TV,PV))
#Output results
results

#########################
# 12.7 For the curious ##
#########################

#.........................................
# 12.7.1 Scheffe's pairwise comparisons ..
#.........................................

#Obtain pairs and their differences
Diff<-numeric(3); Pair<-c(rep(" ",3))
attach(ancova.mod)
Diff[1]<- coefficients[2];Pair[1]<-c("Tr2 - Tr1")
Diff[2]<- coefficients[3];Pair[2]<-c("Tr3 - Tr1")
Diff[3]<-coefficients[2]-coefficients[3]
Pair[3]<-c("Tr2 - Tr3")
detach(ancova.mod)
#Obtain variances and covariance
attach(ancova.sum)
var.Theta2<-sigma^2*cov.unscaled[2,2]
var.Theta3<-sigma^2*cov.unscaled[3,3]
cov.Theta23<-sigma^2*cov.unscaled[2,3]
detach(ancova.sum)
#Obtain standard errors and critical value
SE<-numeric(3)
SE[1]<-sqrt(var.Theta2);SE[2]<-sqrt(var.Theta3)
SE[3]<-sqrt(var.Theta2+var.Theta3-2*cov.Theta23)
crit.val<-sqrt(2*qf(.05,2,26,lower.tail=F))
#Construct interval bounds, test values and p-values
LB<-numeric(3); UB<-numeric(3)
TV<-numeric(3); PV<-numeric(3)
for (i in (1:3)){
     LB[i]<-Diff[i]-crit.val*SE[i]
     UB[i]<-Diff[i]+crit.val*SE[i]
     TV[i]<-Diff[i]^2/(2*SE[i]^2)
     PV[i]<-pf(TV[i],2,11,lower.tail=F)}
#Round down results for readability
for (i in (1:3)){
     Diff[i]<-round(Diff[i],3); SE[i]<-round(SE[i],3)
     LB[i]<-round(LB[i],3);UB[i]<-round(UB[i],3)
     TV[i]<-round(TV[i],3); PV[i]<-round(PV[i],3)}
#store results
results<-data.frame(cbind(Pair,Diff,SE,LB,UB,TV,PV))
#Look at results
results

#.............................
# 12.7.2 The centered model ..
#.............................

#Fit the additive model
options(contrasts=c("contr.sum","contr.poly"))
ancova.mod<-lm(y~Tr+I(x-mean(x)),AncovaData)

#Test constant slope assumption
int.model<-lm(y~Tr+I(x-mean(x))+
                 Tr:I(x-mean(x)),AncovaData)
anova(ancova.mod,int.model)

#Test model significance
red.model<-lm(y~1,AncovaData)
anova(red.model,ancova.mod)

#Test significance of treatment effects
reg.model<-lm(y~I(x-mean(x)),AncovaData)
anova(reg.model,ancova.mod)

###########################################
###########Data generation code############
###########################################

#Initialize variables and "observation" counter
Tr<-integer(30); y<-numeric(30); x<-numeric(30)
k<-1
#Start outer data generation loop for treatments
for (j in 1:3){
	#Get some x-values
	xx<-round(runif(10,5,20),1)
	#assign a mu (can be made random)
	mu<-3.5*j
	#Get random errors
	e<-round(rnorm(10,mean=mu,sd=.5),1)
	#Start inner loop within treatments
	for (i in 1:10){
		Tr[k]<-j
		y[k]<-5+1.2*xx[i]+e[i]
		x[k]<-xx[i]
		k<-k+1}}
#Store results
AncovaData<-data.frame(y,x,Tr)
#Define Tr as a factor
AncovaData$Tr<-factor(AncovaData$Tr)
#Plot to see how the data look
plot(y~x,AncovaData,pch=unclass(Tr),main=NULL)

#Save, if so desired
dump("AncovaData","C:/Docs/RCompanion/Chapter12/Data/Data12x01.R")
