####################### Chapter 13 Script #################
############################################################

#CAUTION: Either restart R or be sure to reset
#the baseline treatment and contrast type to default

#Clear workspace
rm(list=ls(all=T))

################################################################
#Execute this command to duplicate first part of this Chapter ##
################################################################

source("C:/Docs/RCompanion/Chapter13/Data/Data13x01.R")

###################################
# 13.2 Exploratory Data Analysis ##
###################################

#............
# Fig 13.1 ..
#............

#Interaction plots
win.graph(width=6,height=3.5); par(mfrow=c(1,2),cex=.75,ps=10)
with(BlockData,interaction.plot(A,B,y))
with(BlockData,interaction.plot(B,A,y))
par(mfrow=c(1,1))

#Data summaries
summary(BlockData)
with(BlockData,tapply(y,A,summary))
by(BlockData,BlockData$B,summary)

#............
# Fig 13.2 ..
#............

#Stripcharts and boxplots
attach(BlockData)
win.graph(width=6,height=5); par(mfrow=c(2,2),cex=.75,ps=10)
stripchart(y~A,method="jitter",xlab="Response (y)",ylab="Factor (A)")
stripchart(y~B,method="jitter",xlab="Response (y)",ylab="Block (B)")
plot(y~A,ylab="Response (y)",xlab="Factor (A)",horizontal=T)
plot(y~B,ylab="Response (y)",xlab="Block (B)",horizontal=T)
par(mfrow=c(1,1))
detach(BlockData)

####################################
# 13.3 Model Construction and Fit ##
####################################

#Nonadditive model
summary(aov(y~A+B+A:B,BlockData))

#Additive model
summary(aov(y~A+B,BlockData))

#####################
# 13.4 Diagnostics ##
#####################

#Fit the additive model and model summary
Block.mod<-aov(y~A+B,BlockData)
Block.sum<-summary.lm(Block.mod)

#Get relevant diagnostics statistics
e<-residuals(Block.mod)
y.hat<-fitted.values(Block.mod)
r<-e/(Block.sum$sigma*sqrt(1-hatvalues(Block.mod)))
d<-rstudent(Block.mod)

#.......................................
# 13.4.1 Constant variance assumption ..
#.......................................

#............
# Fig 13.3 ..
#............

win.graph(width=6,height=2.5); par(mfrow=c(1,3),cex=.75,ps=10)
with(BlockData,stripchart(r~A,method="stack",
  xlab="A",ylab=expression(hat(r)),vertical=T,las=1))
with(BlockData,stripchart(r~B,method="stack",
  xlab="B",ylab=expression(hat(r)),vertical=T,las=1))
plot(r~y.hat,xlab=expression(hat(y)),ylab=expression(hat(r)),las=1)
par(mfrow=c(1,1))

#Brown-Forsyth test - using function hov
#Load function
library(HH)
hov(e~A,BlockData)
hov(e~B,BlockData)
detach(package:HH);detach(package:lattice)
detach(package:grid);detach(package:multcomp)
detach(package:mvtnorm);detach(package:survival)
detach(package:splines);detach(package:leaps)

#Brown-Forsyth test - using function Anova.bf.test
#Load function
source("C:/Docs/RCompanion/Chapter11/Functions/AnovaBFFTest.R")
with(BlockData,Anova.bf.Ftest(e,A))
with(BlockData,Anova.bf.Ftest(e,B))

#...............................
# 13.4.2 Normality assumption ..
#...............................

#............
# Fig 13.4 ..
#............

win.graph(width=3,height=3.5); par(cex=.75,ps=10)
qqnorm(r,main=NULL);abline(a=0,b=1,lty=3)

#QQ normal probability correlation coefficient test
source("C:/Docs/RCompanion/Chapter6/Functions/qqcortest.R")
qq.cortest(r,0.05)

#Shapiro-Wilk test
shapiro.test(r)

#......................................
# 13.4.3 The independence assumption ..
#......................................

plot(e[-length(e)],e[-1])

#.................................................
# 13.4.4 The presence and influence of outliers ..
#.................................................

#............
# Fig 13.5 ..
#............

win.graph(width=3,height=3.5); par(cex=.75,ps=10)
plot(d,ylab=expression(hat(d)))
m<-ceiling(.05*length(d));df<-Block.mod$df.residual-1
cv<-qt(0.05/(2*m),df,lower.tail=F)
abline(h=c(-cv,0,cv),lty=c(3,1,3))
identify(d)

#Look at influence measures for outliers
inf.tests<-data.frame(influence.measures(Block.mod)[2])
inf.tests[c(10,11),]

###################################################
# 13.5 Pairwise Comparisons of Treatment Effects ##
###################################################

#...................................
# 13.5.1 With a control treatment ..
#...................................

round(Block.sum$coefficients,4)

#......................................
# 13.5.2 Without a control treatment ..
#......................................

#Tukeys HSD test of factor levels
TukeyHSD(Block.mod,"A",ordered=T)

#Comparison of results by altering model formuls
TukeyHSD(aov(y~B+A,BlockData),"A",ordered=T)

####################################
# 13.6 Tukey's Nonadditivity Test ##
####################################

#################################################################
#Execute these command to duplicate later part of this Chapter ##
#################################################################

source("C:/Docs/RCompanion/Chapter13/Data/Data13x02.R")

#............
# Fig 13.6 ..
#............

win.graph(width=6,height=3.5); par(mfrow=c(1,2),cex=.75,ps=10)
with(Block2Data,interaction.plot(A,B,y))
with(Block2Data,interaction.plot(B,A,y))
par(mfrow=c(1,1))

#Tukey's nonadditivity test
#Fit the additive model
add.mod<-lm(y~A+B,Block2Data)
#Extract factor coefficient estimates
A.C<-rep(c(0,add.mod$coefficients[2:3]),each=4)
#Extract block coefficient estimates
B.C<-rep(c(0,add.mod$coefficients[4:6]),3)
#Fit the nonadditive model
nonadd.mod<-lm(y~A+B+I(A.C*B.C),Block2Data)
#Compare models
anova(add.mod,nonadd.mod)

#########################
# 13.7 For the curious ##
#########################

#............................................
# 13.7.1 Bonferroni's pairwise comparisons ..
#............................................

#For convenience
attach(BlockData)
#Initialize counters
n<-length(y);I<-length(levels(A));J<-length(levels(B))
g<-choose(I,2)
#Get cell sizes and means
nA<-tapply(y,A,length);means<-tapply(y,A,mean)
detach(BlockData)
#Initialize storage variables
Pair<-character(g);Diff<-numeric(g)
S.er<-numeric(g)
Tv<-numeric(g);lwr<-numeric(g)
upr<-numeric(g);Pv<-numeric(g)
#Get critical value for intervals
cv<-qt(.05/(2*g),n-I-J+1,lower.tail=F)
#Initialize test counter and begin for loops
k<-1
for (l in 1:(I-1)){
	for (m in (l+1):I){
		#Perform calculations
		Pair[k]<-paste("A",l," vs. ","A",m,sep="")
		Diff[k]<-means[l]-means[m]
		S.er[k]<-Block.sum$sigma*sqrt(1/nA[l]+1/nA[m])
		lwr[k]<-round(Diff[k]-cv*S.er[k],4)
		upr[k]<-round(Diff[k]+cv*S.er[k],4)
		Tv[k]<-abs(Diff[k])/S.er[k]
		Pv[k]<-round(2*g*pt(Tv[k],n-I-J+1,lower.tail=F),3)
		Pv[k]<-ifelse(Pv[k]>1,1,Pv[k]);k<-k+1}}
#Store and output results
Results<-data.frame(Diff,S.er,lwr,upr,Pv,row.names=Pair)
Results

#..................................
# 13.7.2 Creating simulated data ..
#..................................

#......................................
# More than one observation per cell ..
#......................................

#Decide on number of treatments and blocks
I<-3;J<-4
#Set bound for effects parameters
a<-.1;b<-.5;ab<-.1
#Get an overall mean for the response
mu<-runif(1,25,30)
#Generate treatment and blocking effects values
TaoA<-runif(I,-a,a);TaoB<-runif(J,-b,b)
#Generate interaction effects values
TaoAB<-matrix(runif(I*J,-ab,ab),nrow=I,ncol=J)
#Set cell sizes
nij<-matrix(rbinom(I*J,12,.5)+1,nrow=I,ncol=J)
#Get total number of observations
n<-sum(nij)
#Initialize data variables
y<-numeric(n);A<-integer(n);B<-integer(n)
#Get a random sample of error terms
e<-rnorm(n,mean=0,sd=.1)
#Initialize variable index
l<-1
#Start data creation loops
for (i in 1:I){
	for (j in 1:J){
		for (k in 1:nij[i,j]){
		#Get values for the ij cell
		A[l]<-i;B[l]<-j
		y[l]<-mu+TaoA[i]+TaoB[j]+TaoAB[i,j]+e[k]
		#Update variable index and close loops
		l<-l+1}}}
#Store data in a data frame
TestData<-data.frame(y,A,B)
#Define treatment and blocking variables as factors
TestData$A<-factor(TestData$A)
TestData$B<-factor(TestData$B)

#Get interaction plots		
win.graph(width=6,height=3.5); par(mfrow=c(1,2),cex=.75,ps=10)
with(TestData,interaction.plot(A,B,y))
with(TestData,interaction.plot(B,A,y))
par(mfrow=c(1,1))

#Get ANOVA table for nonadditive model
summary(aov(y~A+B+A:B,TestData))

#............................
# One observation per cell ..
#............................

#Simply use the following in the above code
#Set cell sizes
nij<-matrix(rep(1,I*J),nrow=I,ncol=J)





