######################## Chapter 4 Script #################
###########################################################

#Clear workspace
rm(list=ls(all=TRUE))
ls()

#####################
# 4.1 Introduction ##
#####################

#########################################################################
# Data generation code for dataset Salmon
#........................................................................

#Generate a random sample in three groups
y<-c(round(rnorm(500,25,8),1),
      round(rnorm(350,15,4.5),1),round(rnorm(700,7,2.1),1))
#Create a variable to identify groups in y
G<-factor(c(rep(1,500),rep(2,350),rep(3,700)))
#Store the y and G in a data frame and remove y and G from the workspace
Salmon<-data.frame(y,G); rm(y,G)
#########################################################################

#Name variables
names(Salmon)<-c("Weight","Species")
#Assign names to levels
levels(Salmon$Species)<-c("King","Coho","Keta")

################################
# Source file for data used ####
####################################################
#Use this command to duplicate results in chapter ##
#########################################################

source("C:/Docs/RCompanion/Chapter4/Data/Data04x01.R")

#########################################################



#Data summaries
summary(Salmon)
with(Salmon,tapply(Weight,Species,summary))



#################
# 4.3 Boxplots ##
#################

#.....................................
# Fig 4.1 Boxplot of overall Weight ..
#.....................................

#Set a graphics window size and pointsize
win.graph(width=4,height=2.5,pointsize=10)
#Obtain the boxplot of Weight with Salmon
with(Salmon,boxplot(Weight,horizontal=T,xlab="Weight"))

#.......................................
# Fig 4.2 Boxplot of weights of Kings ..
#.......................................

#Set a graphics window size and text pointsize
win.graph(width=4,height=2.5,pointsize=10)
#With Salmon obtain the boxplot of weight for Kings only
with(Salmon,boxplot(Weight[Species=="King"],
              horizontal=T,xlab="Weights of Kings"))

#...........................................
# Fig 4.3 Boxplots of weights by species ...
#...........................................

win.graph(width=6,height=4,pointsize=10)
#Plot weights by species
with(Salmon,boxplot(Weight~Species,horizontal=T,xlab="Weights"))

####################
# 4.4 Histograms ###
####################

#....................................................
# Fig 4.4 Density Histogram of only overall weight ..
#....................................................

win.graph(width=4,height=3,pointsize=10)
#Plot the density histogram of overall weights
with(Salmon,hist(Weight,freq=F))

#.......................................................
# Fig 4.5 Frequency Histogram for King salmon weights ..
#.......................................................

win.graph(width=4,height=3,pointsize=10)
#Plot the frequency histogram of King salmon weights
with(Salmon,hist(Weight[Species=="King"],xlab="Weights",
                           main="Histogram for King Salmon"))

#............................................................
# Histogram for King salmon weights without a main title ....
#............................................................

win.graph(width=4,height=3,pointsize=10)
with(Salmon,hist(Weight[Species=="King"],xlab="Weights",
                           main=NULL))

#....................................................
# Fig 4.6 Multiple histograms in the same window ....
#....................................................

#Open a suitably sized window for three side-by-side plots
win.graph(width=6,height=2)
#Partition the graphics window into one row with three columns
par(mfrow=c(1,3),ps=10)
#Plot each of the density histograms
with(Salmon,hist(Weight[Species=="King"],xlab="Weights",
                       freq=F,main="King Salmon",sub="(a)"))
with(Salmon,hist(Weight[Species=="Coho"],xlab="Weights",
                       freq=F,main="Coho Salmon",sub="(b)"))
with(Salmon,hist(Weight[Species=="Keta"],xlab="Weights",
                       freq=F,main="Keta Salmon",sub="(c)"))
#Reset the graphics window
par(mfrow=c(1,1))

#............................................
# Fig 4.7 Breakpoint options in histograms ..
#............................................

#Open a graphics window, partition into two rows and columns
win.graph(width=6,height=5); par(mfrow=c(2,2),ps=10)
#Plot histogram using Scott's algorithm
with(Salmon,hist(Weight[Species=="King"],xlab="Weights",
           freq=F,breaks="Scott",main="Scott's Algorithm"))
#Plot histogram using the Freedman-Diaconis algorithm
with(Salmon,hist(Weight[Species=="King"],
        xlab="Weights",freq=F,breaks="Freedman-Diaconis",
                        main="Freedman-Diaconis Algorithm"))
#Plot histogram using 6 bins
with(Salmon,hist(Weight[Species=="King"],xlab="Weights",
       freq=F,breaks=6,main="Plot with Six Bins Specified"))
#Plot histogram using given boundary points, 10,12,14,...40
with(Salmon,hist(Weight[Species=="King"],xlab="Weights",
                freq=F,breaks=seq(10,40,2),
                          main="Specified Boundary Points"))
#Reset graphics window
par(mfrow=c(1,1))

##############################################
# 4.5 Density Histogram with density curve ###
##############################################

#............
# Fig 4.8 ...
#............

#Get summary statistics
with(Salmon,summary(Weight[Species=="King"]))

#Open a window
win.graph(width=4,height=3.5,pointsize=10)
#Specify x and y limits
x.lims<-c(14,38); y.lims<-c(0,.12)
#Specify break points
b.pts<-seq(14,38,2)
#Plot the density histogram
with(Salmon,hist(Weight[Species=="King"],freq=F,
#without axes within given x and y limits
                 axes=F,xlim=x.lims,ylim=y.lims,
#and without any titles or axes labels
                    xlab=NULL,ylab=NULL,,main=NULL))

#Obtain a list of at least 100 x-values
x.vals<-seq(14,38,.24)
#Obtain the mean and standard deviations for King weights
mu<-with(Salmon,mean(Weight[Species=="King"]))
s<-with(Salmon,sd(Weight[Species=="King"]))
#Obtain corresponding theoretical densities
y.vals<-dnorm(x.vals,mean=mu,sd=s)
#Superimpose the density curve on the histogram
lines(x.vals,y.vals,lty=2)

#Put in the x-axis, 1
axis(1,seq(14,38,2))
#put in the y-axis, 2
axis(2,seq(0,.12,.03))

#Put in the figure title
title(main="Density histogram and density curve",
#Put in a sub-title
                          sub="King Salmon Sample",
#Label the x- and y-axes
                         xlab="Weight",ylab="Density")

#####################
# 4.6 Stripcharts ###
#####################

#...........
# Fig 4.9 ..
#...........

#Open a window
win.graph(width=5,height=3)
#Scale plotted points by a factor of .75
par(cex=.75)
#Plot a vertical stripchart of weight against species
with(Salmon,stripchart(Weight~Species,vertical=T,
#Plot circles, jittered by an amount of .25
                            pch=1,method="jitter",jitter=.25))

####################################
# 4.7 QQ Normal Probability Plots ##
####################################

#..............................
# Fig 4.10 Multiple qq plots ..
#..............................

#Open a window
win.graph(width=6,height=2.5)
#Format the window and set the scaling factor
par(mfrow=c(1,3),cex=.75)
#Obtain common x- and y-limits for the axes
x.lims<-c(-4,4);y.lims<-c(0,max(Salmon$Weight))
#Plot the three QQ normal probability plots
with(Salmon,qqnorm(Weight[Species=="King"],
    xlim=x.lims,ylim=y.lims,sub="King Salmon",main=NULL))
with(Salmon,qqnorm(Weight[Species=="Coho"],
    xlim=x.lims,ylim=y.lims,sub="Coho Salmon",main=NULL))
with(Salmon,qqnorm(Weight[Species=="Keta"],
    xlim=x.lims,ylim=y.lims,sub="Keta Salmon",main=NULL))
#Reset window settings just in case
par(mfrow=c(1,1),ps=12,cex=1)

#.....................
# Fig 4.11 QQ lines ..
#.....................

#Extract Coho data
Coho.wts<-with(Salmon,Weight[Species=="Coho"])

#Open a window and set parameters
win.graph(width=3,height=3)
par(ps=10, cex=.75)
#Get QQ plot
qqnorm(Coho.wts,main=NULL,sub="Coho Weights")
#Insert the QQ line for reference
qqline(Coho.wts,lty=3)

#..........................................
# Fig 4.12 QQ plots of standardized data ..
#..........................................

#First standardize the Coho data
S.Coho.wts<-(Coho.wts-mean(Coho.wts))/sd(Coho.wts)
#Open a window
win.graph(width=3,height=3);par(ps=10,cex=.75)
#Then plot the QQ plot
qqnorm(S.Coho.wts,main=NULL,
        sub="Standardized Coho Weights")
#Now plot the line y=x as the reference line
abline(a=0,b=1,lty=3)

#########################
# 4.8 Halfnormal plots ##
#########################

#............
# Fig 4.13 ..
#............

#Load package faraway
library(faraway)
#Open a graphics window
win.graph(width=4,height=3.5); par(ps=10,cex=.75)
#Plot the half normal plot
halfnorm(Coho.wts,nlab=2)
#Put in a subtitle
title(sub="Coho Weights")
#Detach package faraway
detach(package:faraway)

#########################
# 4.9 Timeseries plots ##
#########################

x<-c(130,255,301,295,305,395,400,395,463,530,
     200,620,532,650,645,670,551,632,713,523)

#.............
# Fig 4.14 ...
#.............

win.graph(width=4,height=3);par(ps=10)
#Plot the time series graph
plot.ts(x,xlab="Year",ylab="Retreat in feet")

#Formatting time-series data
Retreat<-ts(x,start=1991,end=2010,frequency=1)

plot(Retreat)
plot.ts(Retreat)

#######################
# 4.10 Scatterplots ###
#######################

#...................
# 4.3 duplicated ...
#...................

with(Salmon,plot(Weight~Species,horizontal=T))
methods(plot)

rm(list=ls(all=T))

#............
# Fig 4.15 ..
#............

#Read data03x04 from Chapter 3
MultiNum<-read.table(
 "C:/Docs/RCompanion/Chapter3/Data/Data03x04.txt",header=T)
attach(MultiNum)

#Plot y against x1
win.graph(width=3,height=3);par(ps=10,cex=.75)
plot(x1,y)
plot(y~x1)

#Plotting with type "line"
plot(y~x1,type="l")

detach(MultiNum)

##############################
# 4.11 Matrix scatterplots ###
##############################

#.............
# Fig 4.16 ...
#.............

win.graph(width=6,height=6);par(ps=10,cex=.75)
plot(MultiNum)

with(MultiNum,pairs(cbind(y,x1,x2,x3)))

##########################
# 4.13 For the curious ###
##########################

rm(list=ls(all=T))

#Create a dataset
e<-rnorm(35,mean=3,sd=.25)

################################
# 4.13.1 QQ plot from scratch ##
################################

#Obtain the sample's length
n<-length(e)
#Store the sorted observed quantiles
y.vals<-e[order(e)]
#Compute the plotting positions
p<-(order(y.vals)-0.375)/(n+0.25)
#Compute the theoretical quantiles
x.vals<-qnorm(p)
#Plot the observed values against the expected values
win.graph(width=3,height=3);par(ps=10, cex=.75)
plot(y.vals~x.vals, main="QQ Normal Probability Plot",
         xlab="Expected Quantiles",ylab="Observed Quantiles")

win.graph(width=3,height=3);par(ps=10,cex=.75)
qqnorm(e)

#########################################
# 4.13.2 Halfnormal plot from scratch ###
#########################################

#.............
# Fig 4.18 ...
#.............

#Obtain the sample's length
n<-length(e)
#Store the sorted absolute values of the observed quantiles
y.vals<-sort(abs(e))
#Compute the plotting positions
p<-(order(y.vals)+n-0.125)/(2*n+0.5)
#Compute the theoretical quantiles
x.vals<-qnorm(p)
#Plot the observed values against the expected values
win.graph(width=3,height=3);par(ps=10,cex=.75)
plot(y.vals~x.vals, main="Half-normal Plot",
     xlab="Expected Values",ylab="Absolute Observed Values")

win.graph(width=3,height=3);par(ps=10,cex=.75)
library(faraway)
halfnorm(e)
detach(package:faraway)

##################################
# 4.13.3 Interpreting QQ plots ###
##################################

source("C:/Docs/RCompanion/Chapter4/Data/Data04x01.R")
with(Salmon,qqnorm(Weight))

#Clear workspace
rm(list=ls(all=T))