######################## Chapter 5 Script #################
###########################################################

search()
rm(list=ls(all=T))


#####################################################
# 5.2 Logical Variables, Operators and Statements ###
#####################################################

#.........................................
# 5.2.1 Initializing Logical variables ...
#.........................................

A<-TRUE; B<-FALSE
X<-logical(1); Y<-logical(10)
A<-F; isTRUE(A)

#.............................
# 5.2.2 Logical Operators ....
#.............................

A<-T; isTRUE(A)
A<-c(T,T); isTRUE(A)
A<-T; names(A)<-"A"; isTRUE(A)

#...............................
# 5.2.3 Relational operators ...
#...............................

x<-5
x<6
x!=5
isTRUE(x<=5)

y<-c(1:3)
y<2
y!=2
isTRUE(y<=3)

z<-seq(3,1,-1)
y>z
y!=z
isTRUE((z[1]<=y[1])&(z[2]<=y[2])&(z[3]<=y[3]))
isTRUE((z[1]<=y[1])|(z[2]<=y[2])|(z[3]<=y[3]))

###############################
# 5.3 Conditional statements ##
###############################

#.............
# Preamble ...
#.............

x<- -2:2
round(sqrt(x),3)
asin(x)

#..................
# 5.3.1 If-stop ...
#..................

if (length(x[x<0])>0) stop("Negative data present")

#..................
# 5.3.2 If-Then ...
#..................

if (length(x[x<0])==0) {sqrt(x)}

y<-0:4
if (length(y[y<0])==0){sqrt(y)}

#.......................
# 5.3.3 If-Then-Else ...
#.......................

if (length(x[x<0])==0){
sqrt(x)
	} else stop("Negative data present")

if (length(x[x<0])==0){sqrt(x)
	} else {z<-x[x>=0]
		  print("Negative data excluded",quote=F)
		  sqrt(z)}

#................
# 5.3.4 ifelse ..
#................

ifelse(TRUE,"Apples","Oranges")
ifelse(FALSE,"Apples","Oranges")

(-27)^(1/3)

x<-27; ifelse(x>=0,x^(1/3),-abs(x)^(1/3))
x<- -27; ifelse(x>=0,x^(1/3),-abs(x)^(1/3))

###############
# 5.4 Loops ###
###############

#....................
# 5.4.1 For loops ...
#....................

#Ordered pairs
m<-3;b<-2;xy.pairs<-matrix(nrow=6,ncol=2)
for (i in 0:5){
xy.pairs[i+1,]<-cbind(i,m*i+b)}
xy.pairs

#Geometric series
a<-3; r<-5/6; n<-25; x<-numeric(n)
for (i in 1:n){x[i]<-a*r^(i-1)}
x

#Using character counter lists
for (Data in c("Set 1","Set 2","Set 3")){
	print(paste("Use Data ",Data,sep=""),quote=F)}

#.....................
# 5.4.2 While loops ..
#.....................

#Length of a list
x<-rbinom(rbinom(1,250,.5),150,.5)
#Note: The inner rbinom randomly picks a sample size
#Initialize a length counter
l<-0
#Add one to length counter if next entry is available
while (paste(x[l+1])!=paste(NA)) {l<-l+1}
#Output the result
print(paste("Length of x is",l),quote=F)

##############################
# 5.5 Programming Examples ###
##############################

#...........................................
# 5.5.1 Numerical solution of an equation ..
#...........................................

#...........
# Fig 5.1 ..
#...........

#Graph of function
win.graph(width=4,height=4); par(ps=10)
x<-seq(-.1,2,2.1/100)
y<-3*x*exp(1-2*x)-1
plot(y~x,type="l",axes=F)
axis(1,at=c(seq(-.2,2,.2)),lty=3)
axis(2,at=c(seq(-2,1,.5)),lty=3)
abline(h=0)
abline(v=0)

# Define the function whose zero is to be computed
f<-function(x){return(3*x*exp(1-2*x)-1)}

#Midpoint algorithm
#Initialize the left and right points of the starting interval
a<-0;b<-.5
#Initialize a loop counter
i<-integer(1)
# Start the midpoint algorithm
#Enter loop if f at midpoint is significant to 5 decimal places, and
#as long as the number of iterations is less than 1000
while ((abs(round(f((a+b)/2),5))>=.00001)&(i<=1000)){
       #Pick the half-interval that contains the x-intercept
       if (f(a)*f((a+b)/2)>0){a<-(a+b)/2} else {b<-(a+b)/2}
       #Update the loop counter and signal end of while loop
       i<-i+1}
#Output the result
print(paste("Solution: x = ",round((a+b)/2,3),
       "; Iterations = ",i,sep=""),quote=F)

#.............................
# 5.5.2 A Snazzy histogram ...
#.............................

#Start by a piece-by-piece construction

#Generate a data set
x<-rnorm(50,mean=23,sd=5)

#...........
# Fig 5.2 ..
#...........

#Open a window and create a chart object
win.graph(width=4,height=4);par(ps=10,cex=.75)
chart<-hist(x,plot=FALSE)

attach(chart)
#Obtain number of bins and class width
bins<-length(breaks)
bin.width<-breaks[2]-breaks[1]

#Obtain a sequence of x-values
xmin<-breaks[1]-bin.width; xmax<-breaks[bins]+bin.width
xinc<-(xmax-xmin)/100
xv<-seq(xmin,xmax,xinc)

#Obtain corresponding y-values
yv<-dnorm(xv,mean=mean(x),sd=sd(x))

#Obtain x-axis tick marks
xticks<-seq(xmin,xmax,bin.width)

#Generate y-axis tick marks
#If max(density)>0.5 use 1 dec. place
ydec<-ifelse(max(density)>0.5,1,
   #Else if 0.05<max(density)<=0.5 use 2 dec. places, else use 3
   ifelse((max(density)>0.05)&(max(density)<=0.5),2,3))
#Get the y-increment
yinc<-round(max(density)/5,ydec)
#Get the maximum y-value
ymax<-round(max(density),ydec)+yinc
#Obtain and store the tick mark locations
yticks<-seq(0,ymax,yinc)

#Plot chart
plot(chart,freq=FALSE,axes=FALSE,main=NULL,xlab=NULL,
ylab=NULL,xlim=c(xmin,xmax),ylim=c(0,ymax))

#Plot axes and labels
axis(1,at=xticks); axis(2,at=yticks)
title(main=paste("Histogram and Normal Curve for",
#The deparse function pulls out the actual variable name
deparse(substitute(x))), xlab=deparse(substitute(x)),
ylab="Probability Density")

#Plot density curve
lines(xv,yv,lty=3)

#Put various text and mathematical annotation
mtext(bquote(bar(.(substitute(x)))==.(round(mean(x),2))),
at=max(xv),line=0,adj=1)
mtext(bquote(s==.(round(sd(x),2))),
at=max(xv),line=-1,adj=1)
mtext(bquote(med==.(round(median(x),2))),
at=max(xv),line=-2,adj=1)
detach(chart)


#Code defined as a function

snazzy.hist<-function(x){
win.graph(width=4,height=4);par(mfrow=c(1,1),ps=10,cex=.75)
chart<-hist(x,plot=FALSE)
attach(chart)
#Number of bins
bins<-length(breaks)
#Class width
bin.width<-breaks[2]-breaks[1]
xmin<-breaks[1]-bin.width; xmax<-breaks[bins]+bin.width
xinc<-(xmax-xmin)/100
xv<-seq(xmin,xmax,xinc)
yv<-dnorm(xv,mean=mean(x),sd=sd(x))
xticks<-seq(xmin,xmax,bin.width)
#If max(density)>0.5 use 1 dec. place
ydec<-ifelse(max(density)>0.5,1,
#Else if 0.05<max(density)<=0.5 use 2 decimal places
ifelse((max(density)>0.05)&(max(density)<=0.5),2,
#Else use 3 decimal places
3))
#Get the y-increment
yinc<-round(max(density)/5,ydec)
#Get the maximum y-value
ymax<-round(max(density),ydec)+yinc
#Obtain and store the tick mark locations
yticks<-seq(0,ymax,yinc)
plot(chart,freq=FALSE,axes=FALSE,main=NULL,xlab=NULL,
ylab=NULL,xlim=c(min(xv),max(xv)),ylim=c(0,ymax))
axis(1,at=xticks); axis(2,at=yticks)
title(main=paste("Histogram and Normal Curve for",
#The deparse function pulls out the actual variable name
deparse(substitute(x))), xlab=deparse(substitute(x)),
ylab="Probability Density")
lines(xv,yv,lty=3)
mtext(bquote(bar(.(substitute(x)))==.(round(mean(x),2))),
at=max(xv),line=0,adj=1)
mtext(bquote(s==.(round(sd(x),2))),
at=max(xv),line=-1,adj=1)
mtext(bquote(med==.(round(median(x),2))),
at=max(xv),line=-2,adj=1)
detach(chart)}

##Test run
z<-rnorm(157,mean=13,sd=2.5)
snazzy.hist(z)

#Command to save function snazzy.hist
dump("snazzy.hist","C:/Docs/RCompanion/Chapter5/Functions/snazzyhist.R")

#Command to source the function snazzy.hist
source("C:/Docs/RCompanion/Chapter5/Functions/snazzyhist.R")


