######################## Chapter 6 Script #################
###########################################################

#Clear woskspace
rm(list=ls(all=T))

######################
# 6.1 Introduction ###
######################

#Get the data
source("C:/Docs/RCompanion/Chapter6/Data/Data06x01.R")
ls()

##################################
# 6.2 Exploratory Data Analysis ##
##################################

#.............
# Fig 6.1 ....
#.............

#Get snazzy.hist function and plot the histogram
source("C:/Docs/RCompanion/Chapter5/Functions/snazzyhist.R")
with(SimpleRegData,snazzy.hist(y))

#.............
# Fig 6.2 ....
#.............

#Plot y against x
win.graph(width=3,height=3); par(ps=10,cex=.75)
with(SimpleRegData,plot(y~x,xlab="x",ylab="y"))

#Obtain data summary
summary(SimpleRegData)

###################################
# 6.3 Model construction and fit ##
###################################

#Fitting the model
Simple.mod<-lm(y~x,SimpleRegData)

#Contents of model object
names(Simple.mod)
round(Simple.mod$coefficients,3)

#Model fit
anova(Simple.mod)
(Simple.sum<-summary(Simple.mod))
names(Simple.sum)

#####################
# 6.4 Diagnostics ###
#####################

#Extract necessary diagnostic statistics
e<-residuals(Simple.mod)
y.hat<-fitted.values(Simple.mod)
s<-Simple.sum$sigma
r<-e/s
d<-rstudent(Simple.mod)

#...........................................
# 6.4.1 The constant variance assumption ...
#...........................................

#..............
# Figure 6.3 ..
#..............

win.graph(width=6,height=3.5);par(mfrow=c(1,2),ps=10,cex=.75)
plot(e~y.hat,xlab="Fitted Values",ylab="Residuals")
abline(h=0)
plot(e~SimpleRegData$x,xlab="x",ylab="Residuals")
abline(h=0)
par(mfrow=c(1,1))

#..................
# 6.4.1.1 F-test ..
#..................

#From scratch
GrpA<-residuals(Simple.mod)[SimpleRegData$x<=6]
GrpB<-residuals(Simple.mod)[SimpleRegData$x>6]
df1<-length(GrpA)-1;df2<-length(GrpB)-1
var(GrpA);var(GrpB)
2*pf(var(GrpA)/var(GrpB),df1,df2,lower.tail=F)

#function var.test
var.test(GrpA,GrpB)

var.test(GrpB,GrpA)

#..............................
# 6.4.1.2 Brown-Forsyth test ..
#..............................

#Define function and argument list
#Let e represent the test data, x the sorting variable
#and let cutoff be the desired partitioning value for x.
bf.ttest<-function(e,x,cutoff){
#Get names of test data and sorting variable
dname<-deparse(substitute(e))
xname<-deparse(substitute(x))
#Partition list into two groups and obtain group lengths
e1<-cbind(x,e)[x<=cutoff,2]; e2<-cbind(x,e)[x>cutoff,2]
n<-length(x); n1<-length(e1); n2<-length(e2)
#Obtain group medians
e1.med<-median(e1); e2.med<-median(e2)
#Obtain absolute deviations from medians
d1<-abs(e1-e1.med); d2<-abs(e2-e2.med)
#Obtain means of absolute deviations from medians
d1.ave<-mean(d1); d2.ave<-mean(d2)
#Compute the pooled variance of deviations from the medians
p.var<-(sum((d1-d1.ave)^2)+
                  sum((d2-d2.ave)^2))/(n-2)
#Calculate the test statistic
tbf<-abs(d1.ave-d2.ave)/sqrt(p.var/n1+p.var/n2)
#Compute the p-value using the t-distribution
pv<-round(2*pt(-tbf,n-2),4)
#Store the results in a list
results<-list(statistic=c(t=tbf),parameters=c(df=n-2),
        p.value=pv, method=paste("Brown-Forsyth test ranked by",
          xname," and split at ",xname,"=",cutoff),data.name=dname,
                              alternative=c("Variances are unequal"))
#Format results as hypothesis test class and output
class(results)<-"htest";return(results)}

#Save the source code
dump("bf.ttest","C:/Docs/RCompanion/Chapter6/Functions/RegBFtTest,R")

#Command to source the function
source("C:/Docs/RCompanion/Chapter6/Functions/RegBFtTest.R")

#Apply the function to the given data
with(SimpleRegData,bf.ttest(e,x,6))

#...................................
# 6.4.2 The normality assumption ...
#...................................

#...........
# Fig 6.4 ..
#...........

win.graph(width=3,height=3.5); par(ps=10,cex=.75)
qqnorm(r,main="QQ Plot of Standardized Residuals")
abline(a=0,b=1,lty=3)

qqline(r,lty=3)

#.................................................................
# 6.4.2.1 QQ normality probability correlation coefficient test ..
#.................................................................

#By "hand"
qqplot<-qqnorm(e,plot=FALSE)
with(qqplot,cor(x,y))

#Using the function qq.cortest
source("C:/Docs/RCompanion/Chapter6/Functions/qqcortest.R")
qq.cortest(e,.05)

#.............................
# 6.4.2.2 Shapiro-Wilk test ..
#.............................

shapiro.test(e)

#.....................................
# 6.4.3 The independence assumption ..
#.....................................

plot(e[-18],e[-1])

#................................................
# 6.4.4 The presence and influence of outliers ..
#................................................

#Bonferroni Outlier test

#..............
# Figure 6.5 ..
#..............

win.graph(width=3,height=3.5); par(ps=10,cex=.75)
n<-length(e); m<-ceiling(.05*n); p<-1
cv<-qt(0.05/(2*m),n-p-2,lower.tail=F)
plot(d~y.hat,xlab="Fitted Values",
ylab="Studentized Deleted Residuals",ylim=c(-cv-.5,cv+.5))
abline(h=c(-cv,0,cv),lty=c(3,1,3))

identify(y.hat,d)

#Package car's bonferroni outlier test
library(car)
outlierTest(Simple.mod)
detach(package:car);detach(package:MASS);detach(package:nnet)
detach(package:survival);detach(package:splines)

max(abs(d))
2*pt(max(abs(d)),n-p-2,lower.tail=F)
2*pt(max(abs(d)),n-p-2,lower.tail=F)*n

#########################################
# 6.5 Estimating regression parameters ##
#########################################

#........................
# 6.5.1 One at a time ...
#........................

Simple.sum$coefficients

#Intervals from scratch
#Attach Simple.sum for easy access
attach(Simple.sum)
#Set level of significance
alpha<-0.05
#Extract parameter estimate and standard error
b1<-coefficients[2,1]; sb1<-coefficients[2,2]
#Compute positive critical value
cv<-qt(alpha/2,n-2,lower.tail=F)
#Detach Simple.sum to be safe 
detach(Simple.sum)
#Obtain bounds and print results to screen
lb<-round(b1-cv*sb1,3); rb<-round(b1+cv*sb1,3)
print(paste(lb," < beta_1 < ",rb),quote=F)

#Using a built-in function
confint(Simple.mod,parm="x")

confint(Simple.mod,parm=1)
confint(Simple.mod,parm=1,level=0.99)

#................................
# 6.5.2 Simultaneous intervals ..
#................................

#...........................
# Intervals from scratch ...
#...........................

#Attach Simple.sum for easy access
attach(Simple.sum)
#Set level of significance
alpha<-0.05
#Extract parameter estimates and standard error
b<-coefficients[,1]; sb<-coefficients[,2]
#Compute positive critical value
cv<-qt(alpha/4,n-2,lower.tail=F)
#Detach Simple.sum to be safe 
detach(Simple.sum)
#Obtain bounds and print results to screen
lb<-round(b-cv*sb,3); rb<-round(b+cv*sb,3)
for (j in 1:2) {print(paste(
	lb[j]," < beta_",j-1," < ",rb[j],sep=""),quote=F)}

#Using confint
round(confint(Simple.mod,level=1-0.05/2),3)

###################################################
# 6.6 Confidence Intervals for the Mean Response ##
###################################################

#..................................
# 6.6.1 One-at-a-time intervals ...
#..................................

#................
# From scratch ..
#................

#Set sample size, significance level and point of interest
n<-length(SimpleRegData$y); a<-0.05; x0<-c(1,2.5)
#Attach the summary object for easy access
attach(Simple.sum)
#Extract needed statistics
b<-coefficients[,1]; s<-sigma; XX.Inv<-cov.unscaled
#Detach summary object
detach(Simple.sum)
#Obtain fitted value and critical value
y0<-sum(x0*b); t.val<-qt(a/2,n-2,lower.tail=F)
#Compute standard error for computed fitted value
#with the help of matrix multiplication
s.er<-s*sqrt(t(x0)%*%XX.Inv%*%x0)
#Obtain lower and upper bounds
lb<-round(y0-t.val*s.er,4); ub<-round(y0+t.val*s.er,4)
#Output the results
print(paste("The ",round(100*(1-a)),
      "% Conf. interval at x = ",x0[2],
         " is: (",lb,",",ub,")",sep=""),quote=F)

#..........................
# Using function predict ..
#..........................

x0<-data.frame(x=2.5)
predict(Simple.mod,x0,interval="confidence",level=.95)

#..................................
# 6.6.2 Simultaneous t-intervals ..
#..................................

#..........................
# Using function predict ..
#..........................

#Create input data frame, determine number of intervals
xm<-data.frame(x=c(0.5,1.0,1.5,2.0)); m<-dim(xm)[1]
#And set joint confidence level
a<-.05
#Obtain the confidence intervals
predict(Simple.mod,xm,interval="confidence",level=1-a/m)

#................
# From scratch ..
#................

#Store the points of interest and note numer of intervals
xm<-cbind(rep(1,4),x=c(0.5,1.0,1.5,2.0)); m<-dim(xm)[1]
#Initialize output variables
fitted<-numeric(m); lower<-numeric(m); upper<-numeric(m)
#Note sample size and significance level
n<-length(SimpleRegData$y); a<-0.05
#Attach the summary object for convenience
attach(Simple.sum)
#Extract needed statistics
b<-coefficients[,1]; s<-sigma; XX.Inv<-cov.unscaled
#Detach summary object
detach(Simple.sum)
#Compute the critical value
cv<-qt(a/(2*m),n-2,lower.tail=F)
#Start for-loop to get results
for (k in 1:m){
#Find the fitted values
fitted[k]<-sum(xm[k,]*b)
#Find the standard errors
s.er<-s*sqrt(t(xm[k,])%*%XX.Inv%*%xm[k,])
#Get the lowerbound
lower[k]<-round(fitted[k]-cv*s.er,4)
#Get the upper bound and close the for-loop
upper[k]<-round(fitted[k]+cv*s.er,4)}
#Store the results in a data frame
t.Intervals<-data.frame(cbind(fitted,lower,upper))
t.Intervals

#...................................
# 6.6.3 Simultaneous F-intervals ...
#...................................

#Store the points of interest and note numer of intervals
xm<-cbind(rep(1,4),x=c(0.5,1.0,1.5,2.0)); m<-dim(xm)[1]
#Initialize output variables
fitted<-numeric(m); lower<-numeric(m); upper<-numeric(m)
#Note sample size and significance level
n<-length(SimpleRegData$y); a<-0.05
#Attach the summary object for convenience
attach(Simple.sum)
#Extract needed statistics
b<-coefficients[,1]; s<-sigma; XX.Inv<-cov.unscaled
#Detach summary object
detach(Simple.sum)
#Compute the critical value
cv<-sqrt(2*qf(a,2,n-2,lower.tail=F))
#Start for-loop to get results
for (k in 1:m){
	#Find the fitted values
	fitted[k]<-sum(xm[k,]*b)
	#Find the standard errors
	s.er<-s*sqrt(t(xm[k,])%*%XX.Inv%*%xm[k,])
	#Get the lowerbound
	lower[k]<-round(fitted[k]-cv*s.er,4)
	#Get the upper bound and close the for-loop
	upper[k]<-round(fitted[k]+cv*s.er,4)}
#Store the results in a data frame
F.Intervals<-data.frame(cbind(fitted,lower,upper))
F.Intervals

#............................................
# 6.6.4 Confidence bands using F-intervals ..
#............................................

#Create the matrix Z and get sample size n
attach(SimpleRegData)
a<-floor(min(x)*10)/10;b<-ceiling(max(x)*10)/10
detach(SimpleRegData)
inc<-(b-a)/100; n<-length(SimpleRegData$y)
Z<-cbind(rep(1,101),seq(a,b,inc))
#Initialize output variables and significance level
f<-numeric(101); l<-numeric(101); u<-numeric(101)
a<-0.05
#Attach the summary object for convenience
attach(Simple.sum)
#Extract needed statistics
b<-coefficients[,1]; s<-sigma; XX.Inv<-cov.unscaled
#Detach summary object
detach(Simple.sum)
#Compute the critical value
cv<-sqrt(2*qf(a,2,n-2,lower.tail=F))
#Start for-loop to get results
for (k in 1:101){
     #Get the fitted values and standard errors
     f[k]<-sum(Z[k,]*b)
     s.er<-s*sqrt(t(Z[k,])%*%XX.Inv%*%Z[k,])
     #Get the lower and upper bounds and close the for-loop
     l[k]<-f[k]-cv*s.er;u[k]<-f[k]+cv*s.er}
#Store the results in a data frame
band<-data.frame(f,l,u)

#............
# Fig 6.6 ...
#............

win.graph(width=4,height=3.5); par(ps=10,cex=.75)
matplot(Z[,2],band,lty=c(1,3,3),col=c(1,1,1),
				xlab="x",ylab="E(y)",type="l")
with(SimpleRegData,points(x,y))

##################################################
# 6.7 Prediction Intervals for New Observations ##
##################################################

#..............................................
# 6.7.1 t-interval for a single new response ..
#..............................................

x0<-data.frame(x=2.5)
predict(Simple.mod,x0,interval="prediction")

#..................................
# 6.7.2 Simultaneous t-intervals ..
#..................................

#..........................
# Using function predict...
#..........................

xm<-data.frame(x=c(0.5,1.0,1.5,2.0))
m<-dim(xm)[1]; a<-0.05
predict(Simple.mod,xm,interval="prediction",level=1-a/m)

#................
# From scratch ..
#................

#Store the points of interest and note numer of intervals
xm<-cbind(rep(1,4),x=c(0.5,1.0,1.5,2.0)); m<-dim(xm)[1]
#Initialize output variables
fitted<-numeric(m); lower<-numeric(m); upper<-numeric(m)
#Note sample size and significance level
n<-length(SimpleRegData$y); a<-0.05
#Attach the summary object for convenience
attach(Simple.sum)
#Extract needed statistics
b<-coefficients[,1]; s<-sigma; XX.Inv<-cov.unscaled
#Detach summary object
detach(Simple.sum)
#Compute the critical value
cv<-qt(a/(2*m),n-2,lower.tail=F)
#Start for-loop to get results
for (k in 1:m){
#Find the fitted values
fitted[k]<-sum(xm[k,]*b)
#Find the standard errors
s.er<-s*sqrt(1+t(xm[k,])%*%XX.Inv%*%xm[k,])
#Get the lowerbound
lower[k]<-round(fitted[k]-cv*s.er,4)
#Get the upper bound and close the for-loop
upper[k]<-round(fitted[k]+cv*s.er,4)}
#Store the results in a data frame
t.Intervals<-data.frame(cbind(fitted,lower,upper))
t.Intervals

#...................................
# 6.7.3 Simultaneous F-intervals ...
#...................................

#Store points of interest and number of intervals desired
xm<-cbind(rep(1,4),c(0.5,1.0,1.5,2.0)); m<-dim(xm)[1]
#Initialize output variables, obtain sample size and significance level
fitted<-numeric(m); lower<-numeric(m); upper<-numeric(m)
n<-length(SimpleRegData$y); a<-0.05
#Attach model summary and get model statistics
attach(Simple.sum)
b<-coefficients[,1]; s<-sigma; XX.Inv<-cov.unscaled
detach(Simple.sum)
#Compute the critical value - note the difference from earlier code
cv<-sqrt(m*qf(a,m,n-2,lower.tail=F))
#Start for-loop to get stuff
for (k in 1:m){
     fitted[k]<-sum(xm[k,]*b)
     #Get standard errors - note difference from earlier code
     s.er<-s*sqrt(1+t(xm[k,])%*%XX.Inv%*%xm[k,])
     #Get lower and upper bounds
     lower[k]<-round(fitted[k]-cv*s.er,4)
     upper[k]<-round(fitted[k]+cv*s.er,4)}
F.Intervals<-data.frame(fitted,lower,upper)
F.Intervals

#..........................
# 6.7.4 Prediction bands ..
#..........................

#............
# Fig 6.7 ...
#............

win.graph(width=4,height=3.5); par(ps=10,cex=.75)
X<-data.frame(x=SimpleRegData$x)
p.band<-data.frame(predict(Simple.mod,X,
                  interval="prediction"))
with(SimpleRegData,matplot(x,p.band,lty=c(1,3,3),
		lwd=c(1,2,2),type="l",col=1,xlab="x",ylab="y"))
with(SimpleRegData,points(x,y,pch=18))

#########################
# 6.8 For the curious ###
#########################

#...................................
# 6.8.1 Producing simulated data ...
#...................................

x<-seq(0,12.75,.75)
y<-round(55-1.9*x+rnorm(length(x),mean=0,sd=3.1),1)
#Plot to check for outrageous points
plot(y~x)
#If happy with the data, store and save
SimpleRegData<-data.frame("y"=y,"x"=x)
dump("SimpleRegData","C:/Docs/RCompanion/Chapter6/Data/Data06x01.R")


#..............................
# 6.8.2 The function ci.plot ..
#..............................

win.graph(width=6,height=3.5,pointsize=10)
library(HH)
ci.plot(Simple.mod,conf.level=.95)
detach(package:HH);detach(package:lattice);detach(package:grid)
detach(package:multcomp);detach(package:mvtnorm);detach(package:survival)
detach(package:splines);detach(package:leaps)

#.......................................
# 6.8.3 Brown-Forsyth test revisited ...
#.......................................

#Define function and give argument list
bf.Ftest<-function(e,x,cutoff){
#Get test data and sorting variable names
dname<-deparse(substitute(e)); xname<-deparse(substitute(x))
#Partition list into two groups using the ifelse function
group<-ifelse(x<=cutoff,"A","B")
#Store partitioned data in a data frame
e.list<-data.frame(e,group)
#Compute group medians and deviations from medians
meds<-with(e.list,tapply(e,group,median))
dev<-with(e.list,abs(e-meds[group]))
#Fit to a linear model
info<-anova(lm(dev~group,e.list))
#Extract and store results
results<-list(statistic=c(F=info[1,4]),
   parameters=c(df=info[,1]),p.value=info[1,5],
	method=paste("Brown-Forsyth test using ranking by",
	   xname,"split at",xname," = ",cutoff),data.name=dname,
		alternative=c("Variances are unequal"))
class(results)<-"htest";return(results)}
#Saving command
dump("bf.Ftest","C:/Docs/RCompanion/Chapter6/Functions/RegBFFTest.R")

#Sourcing command for bf.Ftest
source("C:/Docs/RCompanion/Chapter6/Functions/RegBFFTest.R")

with(SimpleRegData,bf.Ftest(e,x,6))

#Sourcing code for bf.ttest
source("C:/Docs/RCompanion/Chapter6/Functions/RegBFtTest.R")

with(SimpleRegData,bf.ttest(e,x,6))

#................
# Function hov ..
#................

Group<-c(rep(1,length(GrpA)),rep(2,length(GrpB)))
Group<-factor(Group)
library(HH)
hov(e~Group)
detach(package:HH);detach(package:lattice);detach(package:grid)
detach(package:multcomp);detach(package:mvtnorm);detach(package:survival)
detach(package:splines);detach(package:leaps)


