######################## Chapter 8 Script #################
###########################################################

#Clear workspace
rm(list=ls(all=T))

##############################################################
#Execute this command to duplicate contents of this chapter ##
##############################################################

source("C:/Docs/RCompanion/Chapter8/Data/Data08x01.R")

##################################
# 8.2 Exploratory Data Analysis ##
##################################

#............
# Fig 8.1 ...
#............

#Load the snazzy.hist function
source("C:/Docs/RCompanion/Chapter5/Functions/snazzyhist.R")
with(MultipleReg,snazzy.hist(y))

#...........
# Fig 8.2 ..
#...........

#Matrix scatterplot
win.graph(width=6,height=6);par(ps=10,cex=.73)
with(MultipleReg,pairs(cbind(y,x1,x2,x3)))

plot(MultipleReg)

###################################
# 8.3 Model Construction and Fit ##
###################################

#Fit the model
Multiple.mod<-lm(y~x1+x2+x3,MultipleReg)

#Parameter estimates for fitted model
round(Multiple.mod$coefficients,4)

#Incorrect way to assess model fit
#Effects of altering order of x's in model formula
anova(lm(y~x1+x2+x3,MultipleReg))
anova(lm(y~x2+x3+x1,MultipleReg))

#Assessing model fit and variable significances
Multiple.sum<-summary(Multiple.mod)
Multiple.sum$coefficients
Multiple.sum

####################
# 8.4 Diagnostics ##
####################

#.................
# Preliminaries ..
#.................

#Extract fitted values
y.hat<-fitted.values(Multiple.mod)

#Extract unstandardized residuals
e<-residuals(Multiple.mod)

#Extract residual standard error
s<-Multiple.sum$sigma
#Compute standardized residuals
r<-e/s

#Extract leverages
h<-hatvalues(Multiple.mod)

#Compute studentized residuals
stud<-e/(s*sqrt(1-h))

#Extract studentized deleted residuals
stud.del<-rstudent(Multiple.mod)

#......................................
# 8.4.1 Constant variance assumption ..
#......................................

#...........
# Fig 8.3 ..
#...........

win.graph(width=6,height=6);par(mfrow=c(2,2),ps=10,cex=.75)
plot(y.hat,stud,xlab="Fitted values",las=1,
      ylab="Studentized residuals",sub="(a)")
abline(h=c(-2,0,2),lty=c(3,1,3))
attach(MultipleReg)
plot(x1,stud,xlab="x1",,las=1,
      ylab="Studentized residuals",sub="(b)")
abline(h=c(-2,0,2),lty=c(3,2,3))
plot(x2,stud,xlab="x2",,las=1,
      ylab="Studentized residuals",sub="(c)")
abline(h=c(-2,0,2),lty=c(3,2,3))
plot(x3,stud,xlab="x3",,las=1,
      ylab="Studentized residuals",sub="(d)")
abline(h=c(-2,0,2),lty=c(3,2,3))
detach(MultipleReg)
par(mfrow=c(1,1))

#...............................................
# 8.4.1.1 F-test for two population variances ..
#...............................................

#Create groups
Group<-with(MultipleReg,list("A"=cbind(e,x1)[x1<=6,1],
            "B"=cbind(e,x1)[x1>6,1]))
#Perform test
var.test(Group$A,Group$B)

#...................................
# 8.4.1.2 The Brown-Forsyth test ...
#...................................

source("C:/Docs/RCompanion/Chapter6/Functions/regBFFTest.R")
with(MultipleReg,bf.Ftest(e,x1,6))


source("C:/Docs/RCompanion/Chapter6/Functions/regBFtTest.R")
with(MultipleReg,bf.ttest(e,x1,6))

#...................................
# 8.4.2 The normality assumption ...
#...................................

#...........
# Fig 8.4 ..
#...........

win.graph(width=3,height=3);par(ps=10,cex=.75)
qqnorm(stud,main=NULL);abline(a=0,b=1)

#............................................................
# 8.4.2.1 QQ probability plot correlation coefficient test ..
#............................................................

source("C:/Docs/RCompanion/Chapter6/Functions/qqcortest.R")
qq.cortest(e,0.05)

#.............................
# 8.4.2.2 Shapiro-Wilk test ..
#.............................

shapiro.test(stud)


#......................................
# 8.4.3 The Independence assumption ...
#......................................

plot(e[-length(e)],e[-1],
	xlab=expression(epsilon[i]),ylab=expression(epsilon[i+1]))


#................................................
# 8.4.4 The presence and influence of outliers ..
#................................................

#...........
# Fig 8.5 ..
#...........

#Set graphics window
win.graph(width=6,height=6); par(mfrow=c(2,2),ps=10,cex=.75)

#..................................................
# 8.4.4.1 Plot (a) - outlying observed responses ..
#..................................................

m<-ceiling(.05*length(y.hat))
plot(stud.del~y.hat,xlab="Fitted values",sub="(a)",
ylab="Studentized deleted residuals",ylim=c(-3,3))
cutoff<-qt(1-0.05/(2*m),length(stud.del)-3-2)
abline(h=c(-cutoff,0,cutoff),lty=c(3,1,3))
#Identify extreme points
identify(y.hat,stud.del)

library(car)
outlierTest(Multiple.mod)
detach(package:car); detach(package:MASS); detach(package:nnet)
detach(package:survival); detach(package:splines)

#...................................................
# 8.4.4.2 Plot (b) - Outliers in the domain space ..
#...................................................

plot(h~y.hat,xlab="Fitted values",las=1,
	ylab="Leverages",sub="(b)",ylim=c(0,.2))
abline(h=2*mean(h),lty=3)
identify(y.hat,h)

#...............................................................
# 8.4.4.3 Plot (c) - Influence on corresponding fitted values ..
#...............................................................

DFFITS<-dffits(Multiple.mod)
plot(abs(DFFITS)~y.hat,xlab="Fitted values",las=1,
	ylab="|DFFITS|",sub="(c)")
abline(h=1,lty=3)
identify(y.hat,abs(DFFITS))

#.....................................................
# 8.4.4.4 Plot (d) - Influence on all fitted values ..
#.....................................................

Cook<-cooks.distance(Multiple.mod)
plot(Cook~y.hat,xlab="Fitted values",las=1,
	ylab="Cook's distances",sub="(d)")
abline(h=qf(0.5,5,36),lty=3)
identify(y.hat,Cook)
par(mfrow=c(1,1))

#............................................
# 8.4.4.5 Influence on parameter estimates ..
#............................................

#...........
# Fig 8.6 ..
#...........

#Extract DFBETAS
DFBETAS<-dfbetas(Multiple.mod)

#Get most extreme absolute DFBETAS value
max(abs(DFBETAS))
#Set window size and properties
win.graph(width=6,height=6); par(mfrow=c(2,2),ps=10,cex=.75)
#Plot (a)
plot(abs(DFBETAS[,1]),ylim=c(0,0.6),sub="(a)",
		ylab=expression(paste(DFBETAS[0])))
#Identify extreme points
identify(abs(DFBETAS[,1]))
#Plot (b)
plot(abs(DFBETAS[,2]),ylim=c(0,0.6),sub="(b)",
		ylab=expression(paste(DFBETAS[1])))
#Identify extreme points
identify(abs(DFBETAS[,2]))
#Plot (c)
plot(abs(DFBETAS[,3]),ylim=c(0,0.6),sub="(c)",
		ylab=expression(paste(DFBETAS[2])))
#Identify extreme points
identify(abs(DFBETAS[,3]))
#Plot (d)
plot(abs(DFBETAS[,4]),ylim=c(0,0.6),sub="(d)",
		ylab=expression(paste(DFBETAS[3])))
#Identify extreme points
identify(abs(DFBETAS[,4]))
par(mfrow=c(1,1))

#...................................
# 8.4.6 Quick influence analysis ...
#...................................

#Extract all influence measures
influence.measures(Multiple.mod)

#To sift through the results
#Get all results
All<-influence.measures(Multiple.mod)
#Separate test results and measures
tests<-data.frame(All[2]);measures<-data.frame(All[1])
#Now sift out the cases that are flagged influential
#First initialize the starter counter
j<-0
#Loop through all 40 cases
for (i in 1:40){
	#Set the test result condition
	is.inf<-FALSE
	#Loop through all 8 columns of All
	for (k in 1:8){
		#Check ith case for influence across 8 columns
		is.inf<-isTRUE((tests[i,k]==TRUE)|is.inf)}
	#If the ith case is influential by one or more measures
	if (is.inf==TRUE){
	#Store the case number
	if (j==0){inf.cases<-i;j<-1} else {inf.cases<-c(inf.cases,i)}}}
#Now pull out only the influential cases
inf.details<-data.frame(measures[inf.cases,])
#Give the columns names
names(inf.details)<-c("dfb0","dfb1","dfb2","dfb3","dffits","cov.r","cook.d","hat")
#Output the findings
round(inf.details,3)
names(tests)<-names(inf.details)
tests[inf.cases,]

#########################################
# 8.5 Estimating regression parameters ##
#########################################

#...................................
# 8.5.1 One-at-a-time t-intervals ..
#...................................

round(confint(Multiple.mod,parm="x2",level=1-.05),3)

#..................................
# 8.5.2 Simultaneous t-intervals ..
#..................................

round(confint(Multiple.mod,
	parm=c("(Intercept)","x1"),level=1-.05/2),3)

#..................................
# 8.5.3 Simultaneous F-intervals ..
#..................................

#Attach summary object for convenience
attach(Multiple.sum)
#Initialize various objects
alpha<-0.05;b<-numeric(4);sb<-numeric(4)
lb<-numeric(4);rb<-numeric(4)
#Calculate "critical" value
cv<-sqrt(df[1]*qf(alpha,df[1],df[2],lower.tail=F))
#Loop through interval computations
for (j in 1:4){
	b[j]<-coefficients[j,1]; sb[j]<-coefficients[j,2]
	lb[j]<-b[j]-cv*sb[j]; rb[j]<-b[j]+cv*sb[j]}
#Store results and name rows appropriately
intervals<-data.frame("Estimate"=b,"Error"=sb,"Lower"=lb,"Upper"=rb)
row.names(intervals)<-c("Intercept","x1","x2","x3")
#Detach summary object to be safe
detach(Multiple.sum)
#Take a look at the results
round(intervals,3)


###################################################
# 8.6 Confidence Intervals for the Mean Response ##
###################################################

#...................................
# 8.6.1 One-at-a-time t-intervals ..
#...................................

#Store the point in a data frame
x<-data.frame(x1=7.36,x2=.41,x3=18.49)
#Obtain the bounds
predict(Multiple.mod,x,interval="confidence")

#...................................
# 8.6.2 Simultaneous t-intervals ...
#...................................

#Store the points
xm<-data.frame(cbind(x1=c(7.7,8.5,1.5,5.8),
	x2=c(2.3,2.6,3.1,2.0),
	x3=c(9.6,13.1,3.4,5.2)))
m<-dim(xm)[1]; alpha<-.05
#Obtain intervals
predict(Multiple.mod,xm,interval="confidence",level=1-alpha/m)

#..................................
# 8.6.3 Simultaneous F-intervals ..
#..................................

#Store the points in a matrix
xm<-cbind(rep(1,4),c(7.7,8.5,1.5,5.8),
c(2.3,2.6,3.1,2.0),c(9.6,13.1,3.4,5.2))
#Initialize the various variables
m<-dim(xm)[1]; fit<-numeric(m)
lwr<-numeric(m); upr<-numeric(m); a<-0.05
#Attach Multiple.sum for easy access
attach(Multiple.sum)
#Extract the various statistics needed
b<-coefficients[,1]; s<-sigma; XtX.inv<-cov.unscaled
#Compute the "critical" value
cv<-sqrt(df[1]*qf(a,df[1],df[2],lower.tail=FALSE))
detach(Multiple.sum)
#Loop through the four points and compute
for (k in 1:m){
	#the fitted value
	fit[k]<-sum(xm[k,]*b)
	#the standard error
	s.er<-s*sqrt(t(xm[k,])%*%XtX.inv%*%xm[k,])
	#Lower and upper bounds
	lwr[k]<-fit[k]-cv*s.er; upr[k]<-fit[k]+cv*s.er}
#Store the results and output for viewing
results<-data.frame(fit,lwr,upr)
round(results,4)

ls()
rm(list=ls(all=T))
source("C:/Docs/RCompanion/Chapter8/Data/Data08x01.R")
Multiple.mod<-lm(y~x1+x2+x3,MultipleReg)
Multiple.sum<-summary.lm(Multiple.mod)

#############################
# 8.7 Prediction Intervals ##
#############################

#...............................................
# 8.7.1 t-intervals for a single new response ..
#...............................................

x<-data.frame(x1=7.36,x2=1.41,x3=18.49)
predict(Multiple.mod,x,interval="prediction")

#...................................
# 8.7.2 Simultaneous t-intervals ...
#...................................

xm<-data.frame(x1=c(7.7,8.5,1.5,5.8),
			x2=c(2.3,2.6,3.1,2.0),
			x3=c(9.6,13.1,3.4,5.2))
m<-4; alpha<-.05
predict(Multiple.mod,xm,
	interval="prediction",level=1-alpha/m)

#..................................
# 8.7.3 Simultaneous F-intervals ..
#..................................

xm<-cbind(rep(1,4),c(7.7,8.5,1.5,5.8),
   c(2.3,2.6,3.1,2.0),c(9.6,13.1,3.4,5.2))
m<-dim(xm)[1]; fit<-numeric(m)
lwr<-numeric(m); upr<-numeric(m); a<-0.05
attach(Multiple.sum)
b<-coefficients[,1]; s<-sigma; XtX.inv<-cov.unscaled
#A different formula is needed here
cv<-sqrt(m*qf(a,m,df[2],lower.tail=F))
detach(Multiple.sum)
for (k in 1:m){
fit[k]<-sum(xm[k,]*b)
#A different formula is need here
s.er<-s*sqrt(1+t(xm[k,])%*%XtX.inv%*%xm[k,])
lwr[k]<-fit[k]-cv*s.er; upr[k]<-fit[k]+cv*s.er}
results<-data.frame(fit,lwr,upr)
round(results,4)

########################
# 8.8 For the Curious ##
########################

#..................................................
# 8.8.1 Fitting and testing a model from scratch ..
#..................................................

y<-as.vector(MultipleReg$y); n<-length(y)
X<-with(MultipleReg,as.matrix(cbind(rep(1,n),x1,x2,x3)))
model.matrix(Multiple.mod)

XtX<-t(X)%*%X; XtX.inv<-solve(XtX)
Multiple.sum$cov.unscaled
XtX.inv

b<-XtX.inv%*%(t(X)%*%y); b
Multiple.mod$coefficients

y.hat<-X%*%b; e<-y-y.hat
SSE<-sum(e^2); MSE<-SSE/(n-4); s<-sqrt(MSE)

I<-as.matrix(rep(1,n))
b.red<-solve(t(I)%*%I)%*%(t(I)%*%y)
y.red<-I%*%b.red; e.red<-y-y.red
SSTo<-sum(e.red^2)
SSR<-SSTo-SSE; MSR<-SSR/3

F.stat<-MSR/MSE
p.value<-pf(F.stat,3,n-4,lower.tail=F)

f.test<-data.frame(
	"df"=c(3,n-4,n-1),
	"SS"=c(SSR,SSE,SSTo),
	"MS"=c(MSR,MSE," "),
	"F.Stat"=c(F.stat," "," "),
	"p.value"=c(round(p.value,4)," "," "),
	row.names=c("Regression","Error","Total"))
f.test
Multiple.sum

#............................................................
# 8.8.2 Joint confidence regions for regression parameters ..
#............................................................

#...........
# Fig 8.7 ..
#...........

win.graph(width=6,height=6);par(ps=10)
j<-2;k<-3 #To be used as parameter identifiers
require(ellipse) #Load package ellipse
#Obtain region bounds and plot
region<-ellipse(Multiple.mod,which=c(j,k),level=.95)
plot(region,type="l",xlab="",main=NULL,ylab="",las=1,
   xlim=c(min(c(0,region[,1])),max(c(0,region[,1]))),
      ylim=c(min(c(0,region[,2])),max(c(0,region[,2]))))
detach(package:ellipse)

#Place horizontal axis label
mtext(bquote(beta[.(j-1)]),side=1,line=3,cex=1.15)
#Place vertical axis label
mtext(bquote(beta[.(k-1)]),side=2,line=3,cex=1.15,las=1)
points(0,0) #Plot the point (0,0)
#Plot the point (beta_1,beta_2)
points(coef(Multiple.mod)[j],coef(Multiple.mod)[k],pch=18)
#Plot lines representing bounds for 1 and 2
abline(v=confint(Multiple.mod)[j,],lty=2)
abline(h=confint(Multiple.mod)[k,],lty=2)
#Put in fancy-pants mathematical annotation
text(-.75,.05,
    expression(frac(paste((hat(bold(theta))-bold(theta)),
      "' [",bold(A)," (",bold(X),"' ",bold(X),
           ")"^-1," ",bold(A),"' ]"^-1," ",
              (hat(bold(theta))-bold(theta))),s^2)
<=2*F(alpha,2,n-p-1)),cex=.9,adj=0)
#Put in legent for plotted points (0,0) and (beta_1,beta_2)
legend(-.75,.5,pch=c(1,18),box.lty=0,
   legend=c("origin: (0,0)",
     expression((list(hat(beta)[j],hat(beta)[k])))))
legend(-.75,.3,lty=3,box.lty=0,
   legend="single parameters bounds")

#.............................................................
# 8.8.3 Bonferroni Confidence regions for the mean response ..
#.............................................................

#...........
# Fig 8.8 ..
#...........

attach(MultipleReg) #For convenience
#Calculate distances of each domain triple from the centroid
dist<-sqrt((x1-mean(x1))^2+(x2-mean(x2))^2+(x3-mean(x3))^2)
#Find the point closest to the centroid
k<-1; while (dist[k]!=min(dist)){k<-k+1}
xk<-c(x1[k],x2[k],x3[k])
#Set the number of intervals and joint significance
n<-length(x1); alpha<-.05
#Obtain upper and lower limits of x1
a<-round(min(x1),2); inc<-(round(max(x1),2)-a)/n
b<-a+(n-1)*inc
#Store coordinates for estimating intervals
xm<-data.frame(x1=seq(a,b,inc),x2=rep(xk[2],n),x3=rep(xk[3],n))
detach(MultipleReg) #To be safe
#Obtain fitted values and bounds
results<-predict(Multiple.mod,xm,interval="confidence",level=1-alpha/n)

win.graph(width=3,height=3); par(mfrow=c(1,1),ps=10,cex=.75)
matplot(xm[,1],results,xlab="x1",ylab="E(y)",
#Specify the type of plot, line type and line colors
	type="l",lty=c(1,3,3),col="black")
#Plot the centroid
with(MultipleReg,points(xk[1],y[x1==xk[1]],pch=18))


########################################
# Generating multiple regression data ##
########################################

x1<-round(runif(40,min=1,max=10),2)
x2<-round(runif(40,min=1,max=5),2)
x3<-round(runif(40,min=1,max=20),2)

y<-round(3-x1+x2+2.5*x3+rnorm(40,mean=0,sd=1),2)

#Check if no negative y's are included
if (length(MultipleReg$y[MultipleReg$y<=0])==0){print("No Negatives")}

#Store in a data frame and tidy up workspace
MultipleReg<-data.frame(y,x1,x2,x3)
rm(x1,x2,x3,y)

#Save the data frame if so desired
dump("MultipleReg","C:/Docs/RCompanion/Chapter8/Data/Data08x01.r")